/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package jorm.rdb.mckoi;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.tuple.api.Tuple;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
// Import clauses required by the object Wsclient2_WsClient2
import jorm.Wsclient2_WsClient2Accessor;
// Imports needed by the rdb mapper
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.io.IOException;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.mapper.rdb.adapter.BasicRdbAdapter;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;

public class Wsclient2_WsClient2Binding
	extends org.objectweb.jonas_ejb.container.JEntitySwitchCS
	implements PBindingCtrl
      {
	/**
	 * The current state of this PBinding object.
	 */
	transient protected byte status;
	/**
	 * The PClassMapping linked to this PBinding object.
	 */
	transient public Wsclient2_WsClient2Mapping pClassMapping;
	/**
	 * The PName of this PBinding object.
	 */
	transient protected PName pname;

	public Wsclient2_WsClient2Binding() {
// nothing
	}

	public Wsclient2_WsClient2Binding(PClassMapping pcm) {
		pClassMapping = (Wsclient2_WsClient2Mapping) pcm;
// nothing
	}


	// IMPLEMENTATION OF METHODS FROM THE PBinding INTERFACE

	public void init(PClassMapping pcm) throws PException {
		pClassMapping = (Wsclient2_WsClient2Mapping) pcm;
		status = PBinding.LIFECYCLE_NOTBOUND;
	    pname = null;
 	}

	public void write(Object conn, PAccessor pa)
		throws PException {
		Wsclient2_WsClient2Accessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Wsclient2_WsClient2Accessor)
			paccessor = (Wsclient2_WsClient2Accessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized WRITE operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String sqlquery;
		    if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Remove");
//Table WSCLIENT2_WSCLIENT2_ : readOnly=false generateUpdate=true colocatedTable=false joins=$tab.joins
                sqlquery = deleteWSCLIENT2_WSCLIENT2_Table(connection, conn, paccessorGen, paccessor);
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [Wsclient2_WsClient2]: problematic SQL query -> {" + sqlquery + "}");
                }
            } else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Insert");
                sqlquery = insertWSCLIENT2_WSCLIENT2_Table(connection, conn, paccessorGen, paccessor);
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [Wsclient2_WsClient2]: problematic SQL query -> {" + sqlquery + "}");
                }
		    } else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Update");
                sqlquery = updateWSCLIENT2_WSCLIENT2_Table(connection, conn, paccessorGen, paccessor);
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [Wsclient2_WsClient2]: problematic SQL query -> {" + sqlquery + "}");
                }
		    }
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se, "SQL problem while writing the DSI.");
		} catch (java.io.IOException ioe) {
			throw new PExceptionIO(ioe, "Stream IO problem while writing the DSI.");
		}
		// Changing the current state
		status = nextState;
	}

	public void read(Object conn, PAccessor pa) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Read");
		Wsclient2_WsClient2Accessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Wsclient2_WsClient2Accessor)
			paccessor = (Wsclient2_WsClient2Accessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet resultSet = null;
        PException resex = null;
		try {
            pstmt = connection.prepareStatement("SELECT WSCLIENT2_WSCLIENT2_.field1_, WSCLIENT2_WSCLIENT2_.field3_, WSCLIENT2_WSCLIENT2_.field2_ FROM WSCLIENT2_WSCLIENT2_ WHERE WSCLIENT2_WSCLIENT2_.field3_ = ?");
            pstmt.setString(1, pname.encodeString());
            resultSet = pstmt.executeQuery();
            if(resultSet.next() == false)
                throw new PExceptionNoDSI(
                    "No entry matching with the following identifier: " + pname);
            PName refname;
            Object fieldval;
            long notnullval;
			if (paccessor != null) {
                paccessor.paSetField1(resultSet.getInt(1));
                fieldval = resultSet.getString(2);
                if (resultSet.wasNull()) {
                    fieldval = null;
                } else {
                }
                paccessor.paSetField3((java.lang.String) fieldval);
                paccessor.paSetField2(resultSet.getInt(3));
			} else if (paccessorGen != null) {
                paccessorGen.paSetIntField(pClassMapping.field1_NAME, resultSet.getInt(1));
                fieldval = resultSet.getString(2);
                if (resultSet.wasNull()) {
                    fieldval = null;
                } else {
                }
                paccessorGen.paSetStringField(pClassMapping.field3_NAME, (java.lang.String) fieldval);
                paccessorGen.paSetIntField(pClassMapping.field2_NAME, resultSet.getInt(3));
			}
		} catch (PException pe) {
			resex = pe;
		} catch (Exception pe) {
			resex = new PExceptionIO(pe, "Problem while reading the DSI.");
		} finally {
		    try {
		        if (resultSet != null) {
			        resultSet.close();
			        resultSet = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
			    	resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		status = nextState;
	}

	public void read(Object conn, PAccessor pa, Object txctx) throws PException {
	    PrefetchCache pc = pClassMapping.getPMapper().getPrefetchCache();
	    if (pc == null || txctx == null) {
	        read(conn, pa);
	        return;
	    }
	    try {
	        Tuple tuple = pc.getPrefetchTuple(pname, pClassMapping, txctx);
	        if (tuple == null) {
	            read(conn, pa);
	            return;
	        }
		    Wsclient2_WsClient2Accessor paccessor = null;
		    PAccessorGen paccessorGen = null;
		    if (pa == null)
			    throw new PExceptionProtocol("Accessor must not be null");
		    else if (pa instanceof Wsclient2_WsClient2Accessor)
			    paccessor = (Wsclient2_WsClient2Accessor) pa;
		    else if (pa instanceof PAccessorGen)
			    paccessorGen = (PAccessorGen) pa;
		    else
			    throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		    byte nextState = PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		    if (nextState==PBinding.LIFECYCLE_ERROR)
			    throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
            Object fval;
            if (paccessor != null) {
                paccessor.paSetField1(tuple.getInt(1));
                paccessor.paSetField2(tuple.getInt(2));
                paccessor.paSetField3(tuple.getString(3));
            } else if (paccessorGen != null) {
                paccessorGen.paSetIntField(pClassMapping.field1_NAME, tuple.getInt(1));
                paccessorGen.paSetIntField(pClassMapping.field2_NAME, tuple.getInt(2));
                paccessorGen.paSetStringField(pClassMapping.field3_NAME, tuple.getString(3));
            }
		    status = nextState;
		} catch (MedorException me) {
		    throw new PExceptionProtocol(me, "Problem while trying to prefetch data from Medor");
		}
    }

	public boolean exist(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Exist");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized EXIST operation, status=" + status);
		status = nextState;
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        PException resex = null;
        boolean exist = false;
		try {
            pstmt = connection.prepareStatement("SELECT WSCLIENT2_WSCLIENT2_.field3_ FROM WSCLIENT2_WSCLIENT2_ WHERE WSCLIENT2_WSCLIENT2_.field3_ = ?");
            pstmt.setString(1, pname.encodeString());
			rs = pstmt.executeQuery();
            exist = rs.next();
		} catch (java.sql.SQLException se) {
			resex = new PExceptionIO(se, "SQL problem while testing the existence of a DSI.");
		} finally {
		    try {
			    if (rs != null) {
			        rs.close();
			        rs = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
					resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		return exist;
	}

	public PName export(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Export");
		return pClassMapping.getPBinder().export(conn, this);
	}

	public PName export(Object conn, Object hints)
		throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Export with hints");
		return pClassMapping.getPBinder().export(conn, this, hints);
	}

	public void unexport(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Unexport");
        pClassMapping.getPBinder().unexport(conn, this.pname, this);
	}

	public void bind(PName pn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Bind");
		// The Pbinding must be unbinded only if the PName has been already
		// defined
		if (pname!=null)
			pClassMapping.getPBinder().unbind(this);
		pClassMapping.getPBinder().bind(pn, this);
	}

	public void unbind() throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Unbind");
		pClassMapping.getPBinder().unbind(this);
	}

	public PClassMapping getPClassMapping() {
		return pClassMapping;
	}

	public PName getPName() {
		return pname;
	}

	public byte getStatus() {
		return status;
	}

	// IMPLEMENTATION OF METHODS FROM THE PBindingCtrl INTERFACE

	public void setPName(PName pn) {
		pname = pn;
	}

	public void setStatus(byte s) {
		status = s;
	}
    private String updateWSCLIENT2_WSCLIENT2_Table(
        Connection connection,
        Object conn,
        PAccessorGen paccessorGen,
        Wsclient2_WsClient2Accessor paccessor)
        throws PException, SQLException, IOException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
            pstmt = connection.prepareStatement(res = "UPDATE WSCLIENT2_WSCLIENT2_ SET field1_ = ?, field3_ = ?, field2_ = ? WHERE field3_ = ?");
                pstmt.setInt(1, paccessor.paGetField1());
                nullField = paccessor.paGetField3();
                if (nullField == null) {
                    pstmt.setNull(2, 12);
                } else {
                    pstmt.setString(2, ((java.lang.String) nullField));
                }
                pstmt.setInt(3, paccessor.paGetField2());
                nullField = paccessor.paGetField3();
                if (nullField == null) {
                    pstmt.setNull(4, 12);
                } else {
                    pstmt.setString(4, ((java.lang.String) nullField));
                }
            } else if (paccessorGen != null) {
            pstmt = connection.prepareStatement(res = "UPDATE WSCLIENT2_WSCLIENT2_ SET field1_ = ?, field3_ = ?, field2_ = ? WHERE field3_ = ?");
                pstmt.setInt(1, paccessorGen.paGetIntField(pClassMapping.field1_NAME));
                nullField = paccessorGen.paGetStringField(pClassMapping.field3_NAME);
                if (nullField == null) {
                    pstmt.setNull(2, 12);
                } else {
                    pstmt.setString(2, ((java.lang.String) nullField));
                }
                pstmt.setInt(3, paccessorGen.paGetIntField(pClassMapping.field2_NAME));
                nullField = paccessor.paGetField3();
                if (nullField == null) {
                    pstmt.setNull(4, 12);
                } else {
                    pstmt.setString(4, ((java.lang.String) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String insertWSCLIENT2_WSCLIENT2_Table(Connection connection,
                                                Object conn,
                                                PAccessorGen paccessorGen,
                                                Wsclient2_WsClient2Accessor paccessor)
        throws PException, SQLException, IOException {

		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO WSCLIENT2_WSCLIENT2_ (field1_, field3_, field2_) VALUES (?, ?, ?)");
                pstmt.setInt(1, paccessor.paGetField1());
                nullField = paccessor.paGetField3();
                if (nullField == null) {
                    pstmt.setNull(2, 12);
                } else {
                    pstmt.setString(2, ((java.lang.String) nullField));
                }
                pstmt.setInt(3, paccessor.paGetField2());
            } else if (paccessorGen != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO WSCLIENT2_WSCLIENT2_ (field1_, field3_, field2_) VALUES (?, ?, ?)");
                pstmt.setInt(1, paccessorGen.paGetIntField(pClassMapping.field1_NAME));
                nullField = paccessorGen.paGetStringField(pClassMapping.field3_NAME);
                if (nullField == null) {
                    pstmt.setNull(2, 12);
                } else {
                    pstmt.setString(2, ((java.lang.String) nullField));
                }
                pstmt.setInt(3, paccessorGen.paGetIntField(pClassMapping.field2_NAME));
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String deleteWSCLIENT2_WSCLIENT2_Table(Connection connection, Object conn, PAccessorGen paccessorGen, Wsclient2_WsClient2Accessor paccessor) throws PException, SQLException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            pstmt = connection.prepareStatement(res = "DELETE FROM WSCLIENT2_WSCLIENT2_ WHERE field3_ = ?");
                nullField = paccessor.paGetField3();
                if (nullField == null) {
                    pstmt.setNull(1, 12);
                } else {
                    pstmt.setString(1, ((java.lang.String) nullField));
                }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
}

















