/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EJBServiceImpl.java,v 1.94 2004/07/02 11:14:30 benoitf Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.container;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.OperationsException;
import javax.management.modelmbean.ModelMBean;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.commons.modeler.ManagedBean;
import org.apache.commons.modeler.Registry;

import org.objectweb.transaction.jta.TransactionManager;

import org.objectweb.jonas_ejb.container.BeanFactory;
import org.objectweb.jonas_ejb.container.Container;
import org.objectweb.jonas_ejb.container.JContainer;
import org.objectweb.jonas_ejb.container.JEntityFactory;
import org.objectweb.jonas_ejb.container.JMdbEndpointFactory;
import org.objectweb.jonas_ejb.container.JMdbFactory;
import org.objectweb.jonas_ejb.container.JSessionFactory;
import org.objectweb.jonas_ejb.container.JStatelessFactory;
import org.objectweb.jonas_ejb.container.PermissionManager;
import org.objectweb.jonas_ejb.container.ThreadPool;
import org.objectweb.jonas_ejb.deployment.api.BeanDesc;
import org.objectweb.jonas_ejb.deployment.api.DeploymentDesc;
import org.objectweb.jonas_ejb.deployment.api.EntityBmpDesc;
import org.objectweb.jonas_ejb.deployment.api.EntityCmpDesc;
import org.objectweb.jonas_ejb.deployment.api.SessionStatefulDesc;
import org.objectweb.jonas_ejb.deployment.api.SessionStatelessDesc;
import org.objectweb.jonas_ejb.deployment.lib.wrapper.EjbManagerWrapper;

import org.objectweb.jonas_lib.deployment.api.DeploymentDescException;
import org.objectweb.jonas_lib.deployment.work.WorkCleaner;
import org.objectweb.jonas_lib.deployment.work.CleanerException;
import org.objectweb.jonas_lib.deployment.work.JarCleanTask;
import org.objectweb.jonas_lib.deployment.work.DeployerLog;
import org.objectweb.jonas_lib.deployment.work.DeployerLogException;
import org.objectweb.jonas_lib.deployment.work.FileManager;
import org.objectweb.jonas_lib.loader.EjbJarClassLoader;

import org.objectweb.jonas_jms.api.JmsManager;

import org.objectweb.jonas.common.JModule;
import org.objectweb.jonas.common.JProp;
import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.ear.EarServiceImpl;
import org.objectweb.jonas.jms.JmsService;
import org.objectweb.jonas.jmx.J2eeObjectName;
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.jtm.TransactionService;
import org.objectweb.jonas.management.JonasMBeanTools;
import org.objectweb.jonas.naming.CompNamingContext;
import org.objectweb.jonas.naming.NamingManager;
import org.objectweb.jonas.security.SecurityService;
import org.objectweb.jonas.server.LoaderManager;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.service.ServiceManager;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Implementation of the EJB Container Service for jonas. This class works only
 * with jonas_ejb classes.
 * @author Philippe Coq
 * @author Jeff Mesnil (Security)
 * @author Markus Karg (Novell port)
 * @author Christophe Ney (for making easier Enhydra integration)
 * @author Adriana Danes (complete management methods)
 * @author Florent Benoit & Ludovic Bert (Ear service, deployJars, undeployJars)
 * @author Benjamin Bonnet (max thread pool size)
 * @author Michel-Ange Anton (JSR77 MBean : EJBModule)
 * @author Adriana Danes (JSR77)
 */
public class EJBServiceImpl extends AbsServiceImpl implements EJBService, EJBServiceImplMBean {

    // loggers
    private static Logger servlog = null;

    private static Logger loaderlog = null;

    // Transaction manager reference
    // (tm can be accessed directly from everywhere in the package.)
    private TransactionManager tm = null;

    // JMS manager reference
    private JmsManager jms = null;

    // JMX Server
    private MBeanServer mbeanServer = null;

    // Security service reference
    private SecurityService securityService = null;

    /**
     * Reference to the cleaner
     */
    private static WorkCleaner workCleaner = null;

    // List of the ejb names to load when starting the EJB Container Service
    private Vector ejbNames = new Vector();

    // We have a Container by ejb-jar file.
    private Vector containers = new Vector();

    // default thread pool size
    int threadPoolSize = 0;

    // default max thread pool size
    int maxthreadPoolSize = ThreadPool.DEFAULT_POOL_MAX_SIZE;

    /**
     * The name of the JONAS_BASE directory.
     */
    protected static final String JONAS_BASE = JProp.getJonasBase();

    /**
     * The name of the directory containing the ejb-jar files.
     */
    protected static final String EJBJARS_DIR = JONAS_BASE + File.separator + "ejbjars";

    /**
     * List of autoloaded directories.
     */
    private ArrayList autoloadDirectories = new ArrayList();

    // Service configuration properties
    public static final String AUTOLOADDIR = "jonas.service.ejb.autoloaddir";

    public static final String MDBTHREADPOOLSIZE = "jonas.service.ejb.mdbthreadpoolsize";

    public static final String MDBMAXTHREADPOOLSIZE = "jonas.service.ejb.mdbmaxthreadpoolsize";

    public static final String PARSINGWITHVALIDATION = "jonas.service.ejb.parsingwithvalidation";

    public static final String DESCRIPTORS = "jonas.service.ejb.descriptors";

    public static final String CLASS = "jonas.service.ejb.class";

    // Management properties
    public static final String BMP = "Bean-Managed Persistency";

    public static final String CMP = "Container-Managed Persistency";

    // applications parent classloader
    private ClassLoader appsClassLoader = null;

    /**
     * The name of the working directory.
     */
    private static final String WORK_DIR = JProp.getWorkDir();

    /**
     * The name of the working ejbjars directory.
     */
    private static final String WORK_EJBJARS_DIR = WORK_DIR + File.separator + "ejbjars";

    /**
     *
     */
    private static String ejbjarsDir = null;

    /**
     * Reference on the DeployerLog which is the class that manage the accesses
     * to the log file (to remove the jar).
     */
    private DeployerLog jarDeployerLog = null;

    // -------------------------------------------------------------------
    // JOnAS Service Implementation
    // -------------------------------------------------------------------

    /**
     * Init the Service. Configuration information is passed thru a Context
     * object.
     */
    public void doInit(Context ctx) throws ServiceException {
        servlog = Log.getLogger(Log.JONAS_SERVER_PREFIX);
        loaderlog = Log.getLogger(Log.JONAS_LOADER_PREFIX);

        servlog.log(BasicLevel.DEBUG, "");

        // get apps ClassLoader
        try {
            LoaderManager lm = LoaderManager.getInstance();
            appsClassLoader = lm.getAppsLoader();
        } catch (Exception e) {
            servlog.log(BasicLevel.ERROR, "Cannot get the Applications ClassLoader from EJB Container Service: " + e);
            throw new ServiceException("Cannot get the Applications ClassLoader from EJB Container Service", e);
        }

        ServiceManager serviceManager = null;
        try {
            serviceManager = ServiceManager.getInstance();
        } catch (Exception e) {
            servlog.log(BasicLevel.ERROR, "Cannot initialize the EJB Container Service: " + e);
            throw new ServiceException("Cannot initialize the EJB Container Service", e);
        }
        // Get the used services
        try {
            // Get the Transaction Manager via the jtm service
            TransactionService jtmService = (TransactionService) serviceManager.getTransactionService();
            tm = jtmService.getTransactionManager();
        } catch (ServiceException e) {
            servlog.log(BasicLevel.ERROR, "Cannot initialize the EJB Container Service: " + e);
            throw new ServiceException("Cannot initialize the EJB Container Service", e);
        }

        try {
            // Get the Security service
            securityService = (SecurityService) serviceManager.getSecurityService();
        } catch (ServiceException se) {
            // The Security service may not be started
            securityService = null;
        }

        try {
            // Get the JMS manager via the JMS service
            JmsService jmsService = (JmsService) serviceManager.getJmsService();
            jms = jmsService.getJmsManager();
        } catch (ServiceException e) {
            // the JMS service may not be started
            jms = null;
        }

        try {
            // Get the JMX Server via JMX Service
            JmxService jmxService = (JmxService) serviceManager.getJmxService();
            mbeanServer = jmxService.getJmxServer();
        } catch (ServiceException e) {
            // the JMX service may not be started
            mbeanServer = null;
        }

        // Set the XML parsing mode to no validation
        String parsingMode = "false";
        try {
            parsingMode = (String) ctx.lookup(PARSINGWITHVALIDATION);
        } catch (NamingException e) {
            // No problem if there is no value for 'parsingwithvalidation'
            // (false by default)
        }
        if ("false".equalsIgnoreCase(parsingMode)) {
            EjbManagerWrapper.setParsingWithValidation(false);
            servlog.log(BasicLevel.DEBUG, "XML parsing without validation");
        } else {
            servlog.log(BasicLevel.DEBUG, "XML parsing with validation");
        }

        // Init the ejb names to be loaded when starting
        String descsValue = null;
        try {
            descsValue = (String) ctx.lookup(DESCRIPTORS);
        } catch (NamingException e) {
            // No problem if there is no value for 'descriptors' (no ejb to
            // load)
        }
        if (descsValue != null) {
            StringTokenizer st = new StringTokenizer(descsValue, ",");
            while (st.hasMoreTokens()) {
                String fileName = st.nextToken().trim();
                ejbNames.add(fileName);
            }
        }

        // Add the wars of the jonas.service.ejb.autoloaddir property
        String dirValue = null;
        ArrayList autoDirs = new ArrayList();
        try {
            dirValue = (String) ctx.lookup(AUTOLOADDIR);
        } catch (NamingException e) {
            // No problem if there is no value for 'autoloaddir'
            //(no ejb to load)
        }
        if (dirValue != null) {
            StringTokenizer st = new StringTokenizer(dirValue, ",");
            while (st.hasMoreTokens()) {
                String dirName = st.nextToken().trim();
                addEjbjars(dirName);
                autoDirs.add(dirName);
            }
        }
        // Build autoload directories
        File oFile;
        for (int i = 0; i < autoDirs.size(); i++) {
            try {
                oFile = new File(EJBJARS_DIR, autoDirs.get(i).toString());
                if (oFile.exists()) {
                    autoloadDirectories.add(oFile.getCanonicalPath());
                } else {
                    oFile = new File(autoDirs.get(i).toString());
                }
            } catch (Exception e) {
                String err = "Error when trying to verify Ejbjar autoload directory : " + autoDirs.get(i);
                servlog.log(BasicLevel.ERROR, err + " " + e.getMessage());
            }
        }

        // MDB thread pool size
        try {
            String plz = (String) ctx.lookup(MDBTHREADPOOLSIZE);
            threadPoolSize = (new Integer(plz)).intValue();
        } catch (NamingException e) {
            // default value will be used.
        }
        // MAX thread pool size
        try {
            String mxplz = (String) ctx.lookup(MDBMAXTHREADPOOLSIZE);
            maxthreadPoolSize = (new Integer(mxplz)).intValue();
        } catch (NamingException e) {
            // default value will be used.
        }
        servlog.log(BasicLevel.DEBUG, "EJB Container Service initialized");

    }

    /**
     * Start the Service Initialization of the service is already done.
     */
    public void doStart() throws ServiceException {

        ejbjarsDir = WORK_EJBJARS_DIR + File.separator + getJonasServerName();

        URL ejbjarsUrl = null;
        try {
            ejbjarsUrl = new File(ejbjarsDir).toURL();
        } catch (MalformedURLException mue) {
            throw new ServiceException("Error when trying to get the URL of the jonasroot/apps directory", mue);
        }

        File fLog = new File(ejbjarsUrl.getFile() + File.separator + getJonasServerName() + ".log");
        if (!fLog.exists()) {
            try {
                //create
                fLog.getParentFile().mkdirs();
                fLog.createNewFile();
            } catch (IOException e) {
                throw new ServiceException("cannot create the log file" + fLog);
            }
        }

        //get the logger
        try {
            jarDeployerLog = new DeployerLog(fLog);
        } catch (DeployerLogException e) {
            throw new ServiceException("Can not get an EarDeployerLog", e);
        }

        // create the jar deployment work file control task
        JarCleanTask jarCleanTask = new JarCleanTask(ejbjarsUrl, jarDeployerLog);

        //get the cleaner ref
        workCleaner = WorkCleaner.getInstance();

        // add the jar deployment work file control task
        try {
            workCleaner.registerTask(jarCleanTask);
        } catch (CleanerException ce) {
            throw new ServiceException("Cannot register the JAR clean task", ce);

        }

        // force the cleaning immediately
        workCleaner.executeTasks();

        // Creates all predefined containers
        // 1 container per ejbjar file.
        String fileName = null;
        Context contctx = null;
        for (int i = 0; i < ejbNames.size(); i++) {
            fileName = (String) ejbNames.elementAt(i);
            try {
                contctx = new CompNamingContext(fileName);
                contctx.rebind("filename", fileName);
            } catch (NamingException ne) {
                servlog.log(BasicLevel.WARN, "Cannot create container for " + fileName
                        + " because of a NamingException : " + ne.toString());
            }

            try {
                createContainer(contctx);
            } catch (Exception e) {
                servlog.log(BasicLevel.WARN, "Cannot create container for " + fileName + " because of Exception : "
                        + e.toString());
                // Close naming context
                try {
                    contctx.close();
                } catch (NamingException nne) {
                }
            }
        }

        // Register EJBService MBean : EJBServiceImplMBean
        if (mbeanServer != null) {
            try {
                mbeanServer.registerMBean(this, JonasObjectName.ejbService());
            } catch (MBeanException mbe) {
                // registration failed because a MBeanException
                servlog.log(BasicLevel.WARN, "Cannot register the EJB Container Service in the MBean Server.");
                servlog.log(BasicLevel.WARN, "Regstration failure cause : " + mbe.getTargetException().toString());
            } catch (OperationsException me) {
                // registration failed because a OperationsException JMX
                // Exception
                servlog.log(BasicLevel.WARN, "Cannot register the EJB Container Service in the MBean Server.");
                servlog.log(BasicLevel.WARN, "Regstration failure cause : " + me.toString());
            }
        }
    }

    /**
     * Stop the service: - Remove all JOnAS Containers. - Unbinds all the
     * EJBHome names from JNDI
     */
    public void doStop() {

        Enumeration lc = containers.elements();
        while (lc.hasMoreElements()) {
            Container cont = (Container) lc.nextElement();
            removeContainer(cont);
        }
        if (mbeanServer != null) {
            try {
                // unregister EJBService MBean : EJBServiceImplMBean
                mbeanServer.unregisterMBean(JonasObjectName.ejbService());
            } catch (Exception e) {
                servlog.log(BasicLevel.ERROR, "Cannot stop the EJB Container Service: " + e);
            }
        }

        servlog.log(BasicLevel.DEBUG, "EJB Container Service stopped");
    }

    // -------------------------------------------------------------------
    // EJBService Implementation
    // -------------------------------------------------------------------

    /**
     * Create a JOnAS Container for all the beans that are described in a .xml
     * file, or belong to .jar file.
     * @return The ObjectName of the MBean associated to the container (i.e. to
     *         the deployed module)
     * @throws Exception if an error occur during the creation of the container.
     */
    public String createContainer(Context ctx) throws Exception {
        servlog.log(BasicLevel.DEBUG, "");

        // Get the file name
        String fileName = (String) ctx.lookup("filename");
        boolean isEjbJarFile = fileName.toLowerCase().endsWith(".jar");

        // check if the file exists
        // In case of the name is a ejb-jar file name, check also in
        // the JONAS_BASE/ejbjars directory
        File f = null;
        try {
            f = new File(fileName).getCanonicalFile();
            if (!f.exists() && isEjbJarFile) {
                String ejbjarFileName = EJBJARS_DIR + File.separator + fileName;
                f = new File(ejbjarFileName).getCanonicalFile();
                if (!f.exists()) {
                    servlog.log(BasicLevel.ERROR, "createContainer: " + fileName + " not found");
                    throw new NamingException(fileName + " not found");
                }
            }
        } catch (IOException e) {
            String err = "Invalid ejbjar file name '" + fileName;
            servlog.log(BasicLevel.ERROR, err);
            throw new Exception(err + e.getMessage());
        }

        // Force fileName with slash to avoid conflict under Windows (slash and
        // back-slash)
        try {
            fileName = f.toURL().getPath();
        } catch (MalformedURLException e) {
            servlog.log(BasicLevel.ERROR, "Invalid ejb-jar file name '" + fileName + "': " + e);
        }

        if (f.isFile()) {
            if (!isEjbJarFile && !fileName.toLowerCase().endsWith(".xml")) {
                throw new ServiceException("The ejbjar to deploy is not a jar file nor an xml file");
            }
        }

        // Check if container already exists
        if (getContainer(fileName) != null) {
            servlog.log(BasicLevel.ERROR, "createContainer: " + fileName + " already exists");
            throw new Exception("Container already exists");
        }

        // In order to avoid lock problem after undeploy
        // the jar file is copied under the working directory before its loading
        // ! only if the ejbjars isn't included in an ear
        boolean isInEar = true;
        URLClassLoader earCL = null;
        try {
            earCL = (URLClassLoader) ctx.lookup("earClassLoader");
        } catch (NamingException ne) {
            // do nothing
        }
        isInEar = (!(earCL == null));

        String wkFileName = null;

        if (!isInEar) {
            // Get the destination file
            wkFileName = ejbjarsDir + File.separator
                    + FileManager.fileToTimeStampDir(new File(fileName).toURL(), ".jar");

            servlog.log(BasicLevel.DEBUG, "filename=" + fileName);
            servlog.log(BasicLevel.DEBUG, "wkFilename=" + wkFileName);

            // copy the file
            FileManager.copyFile(fileName, wkFileName);

            // add an entry in the log file
            try {
                jarDeployerLog.addEntry(new File(fileName), new File(wkFileName));
            } catch (DeployerLogException e) {
                String err = "Error while adding the " + fileName + " entry in the log file";
                servlog.log(BasicLevel.ERROR, err + " : " + e.getMessage());
                throw new Exception(err, e);
            }

            servlog.log(BasicLevel.DEBUG, "Create wk file :" + wkFileName);

        } else {
            // in this case, no needs to copy the file
            wkFileName = fileName;
            servlog.log(BasicLevel.DEBUG, "No needs wk file :" + wkFileName);
        }

        // Classloader
        URL[] url = null;
        try {
            url = new URL[1];
            url[0] = (new File(wkFileName)).toURL();
        } catch (MalformedURLException e) {
            servlog.log(BasicLevel.ERROR, "Invalid ejb-jar file name '" + fileName + "': " + e);
        }

        // Get the classloaders
        URLClassLoader ejbClassLoader = null;
        URLClassLoader earClassLoader = null;
        try {
            earClassLoader = (URLClassLoader) ctx.lookup("earClassLoader");
            ejbClassLoader = (URLClassLoader) ctx.lookup("ejbClassLoader");
            loaderlog.log(BasicLevel.DEBUG, "earClassLoader=" + earClassLoader);
        } catch (NamingException ne) {
            // no ear classloader => ejb application case (not ear case).
            if (isEjbJarFile) {
                ejbClassLoader = new EjbJarClassLoader(url, appsClassLoader);
            } else {
                ejbClassLoader = (URLClassLoader) appsClassLoader;
            }
            loaderlog.log(BasicLevel.DEBUG, "parent Loader=" + appsClassLoader);
        }
        loaderlog.log(BasicLevel.DEBUG, "ejbClassLoader=" + ejbClassLoader);

        // Get the deployment descriptor from file
        DeploymentDesc dd = null;
        try {
            // ejb-jar file
            dd = EjbManagerWrapper.getDeploymentDesc(url[0], ejbClassLoader, earClassLoader);
        } catch (DeploymentDescException e) {
            servlog.log(BasicLevel.ERROR, "Cannot read the deployment descriptors '" + fileName + "': " + e);
            throw new ServiceException(e.getMessage(), e);
        }

        // container name
        String cname = buildEJBModuleName(fileName);
        if (cname == null) {
            //use displayName
            cname = "EJB container ";
            if (dd.getDisplayName() != null) {
                cname += dd.getDisplayName();
            }
        }

        // Create the Container Object
        JContainer cont = new JContainer(cname, fileName, wkFileName, ejbClassLoader);
        cont.setContainerNaming(NamingManager.getInstance());
        cont.setTransactionManager(tm);
        cont.setPrincipalFactory(new PrincipalFactoryImpl());
        cont.setJmsManager(jms);

        // Set the name of the ear application containing this container
        // in the case of an ear application.
        String earFileName = null;
        try {
            earFileName = (String) ctx.lookup("earFileName");
            cont.setEarFileName(earFileName);
        } catch (NamingException ne) {
        }

        // unset the security if no security service
        if (securityService == null) {
            servlog.log(BasicLevel.DEBUG, "EJB Container Service: working without Security Service");
            cont.setSecurity(false);
        } else {
            PermissionManager permissionManager = null;
            try {
                permissionManager = new PermissionManager(dd, fileName);
                permissionManager.translateEjbDeploymentDescriptor();
                permissionManager.commit();
            } catch (Exception e) {
                servlog.log(BasicLevel.ERROR, "Can't build permission manager object for the ejbjar '" + fileName
                        + "' : " + e.getMessage());
            }
            cont.setPermissionManager(permissionManager);
        }

        // Set thread pool size if it has been set in properties
        // Note that this ThreadPool is used only for MDB (jms), but it could
        // be used for other purpose later.
        if (threadPoolSize > 0) {
            cont.setThreadPoolSize(threadPoolSize);
        }
        cont.setMaxThreadPoolSize(maxthreadPoolSize);

        String domainName = getDomainName();
        String serverName = getJonasServerName();
        String j2eeAppName = EarServiceImpl.buildJ2eeApplicationName(earFileName);
        Registry oRegistry = JonasMBeanTools.getRegistry();
        EJBModule ejbModuleMBean = null;
        ObjectName onEjbModule = null;

        // MBeans to register if the deployment is successful
        Map mbeansToRegister = new HashMap();

        // EJBModule MBean registering
        if (mbeanServer != null) {
            // Use the conatiner name (the 'name' constructor parameter of
            // JContainer class)
            // as the "name" keyProperty for the J2ee ObjectName construction
            onEjbModule = J2eeObjectName.getEJBModule(domainName, serverName, j2eeAppName, cname);

            ejbModuleMBean = new EJBModule(onEjbModule.toString(), cont, fileName, cname);
            ejbModuleMBean.setDeploymentDescriptor(dd.getXmlContent());
            ejbModuleMBean.setJonasDeploymentDescriptor(dd.getJOnASXmlContent());

            // Use Jakarta Common Modeler API
            ManagedBean oManaged = oRegistry.findManagedBean("EJBModule");
            ModelMBean oMBean = oManaged.createMBean(ejbModuleMBean);
            mbeansToRegister.put(oMBean, onEjbModule);
        }

        // Process each bean from the DD
        // LATER: give it its deployment desc to do the job ?
        BeanDesc[] beans = dd.getBeanDesc();
        for (int i = 0; i < beans.length; i++) {
            BeanFactory bf = cont.addBean(beans[i]);
            if (mbeanServer != null) {
                try {
                    // register MBean for this ejb
                    String beanName = beans[i].getEjbName();
                    if ((beans[i] instanceof EntityBmpDesc) || (beans[i] instanceof EntityCmpDesc)) {
                        JEntityFactory _bf = (JEntityFactory) bf;
                        ObjectName onEntityBean = J2eeObjectName.getEntityBean(domainName, cname, serverName,
                                j2eeAppName, beanName);
                        EntityBean entityBeanMBean; // new EntityBean
                                                    // declaration
                        if (beans[i] instanceof EntityBmpDesc) {
                            entityBeanMBean = new EntityBean(onEntityBean.toString(), _bf, BMP);
                        } else {
                            entityBeanMBean = new EntityBean(onEntityBean.toString(), _bf, CMP);
                        }
                        ManagedBean oManaged = oRegistry.findManagedBean("EntityBean");
                        ModelMBean oMBean = oManaged.createMBean(entityBeanMBean);
                        mbeansToRegister.put(oMBean, onEntityBean);
                        // Update the EJBModule
                        ejbModuleMBean.addEjb(onEntityBean.toString());
                    } else if (beans[i] instanceof SessionStatefulDesc) {
                        JSessionFactory _bf = (JSessionFactory) bf;
                        ObjectName onStatefulSessionBean = J2eeObjectName.getStatefulSessionBean(domainName, cname,
                                serverName, j2eeAppName, beanName);
                        StatefulSessionBean sessionBeanMBean = new StatefulSessionBean(
                                onStatefulSessionBean.toString(), _bf);
                        ManagedBean oManaged = oRegistry.findManagedBean("StatefulSessionBean");
                        ModelMBean oMBean = oManaged.createMBean(sessionBeanMBean);
                        mbeansToRegister.put(oMBean, onStatefulSessionBean);
                        // Update the EJBModule
                        ejbModuleMBean.addEjb(onStatefulSessionBean.toString());
                    } else if (beans[i] instanceof SessionStatelessDesc) {
                        JStatelessFactory _bf = (JStatelessFactory) bf;
                        ObjectName onStatelessSessionBean = J2eeObjectName.getStatelessSessionBean(domainName, cname,
                                serverName, j2eeAppName, beanName);
                        StatelessSessionBean sessionBeanMBean = new StatelessSessionBean(onStatelessSessionBean
                                .toString(), _bf);
                        ManagedBean oManaged = oRegistry.findManagedBean("StatelessSessionBean");
                        ModelMBean oMBean = oManaged.createMBean(sessionBeanMBean);
                        mbeansToRegister.put(oMBean, onStatelessSessionBean);
                        // Update the EJBModule
                        ejbModuleMBean.addEjb(onStatelessSessionBean.toString());
                    } else {
                        ObjectName onMessageDrivenBean = J2eeObjectName.getMessageDrivenBean(domainName, cname,
                                serverName, j2eeAppName, beanName);
                        MessageDrivenBean messageDrivenBeanMBean = null;
                        if (bf instanceof JMdbFactory) {
                            JMdbFactory _bf = (JMdbFactory) bf;
                            messageDrivenBeanMBean = new MessageDrivenBean(onMessageDrivenBean.toString(), _bf);
                        } else {
                            JMdbEndpointFactory _bf = (JMdbEndpointFactory) bf;
                            messageDrivenBeanMBean = new MessageDrivenBean(onMessageDrivenBean.toString(), _bf);
                        }
                        ManagedBean oManaged = oRegistry.findManagedBean("MessageDrivenBean");
                        ModelMBean oMBean = oManaged.createMBean(messageDrivenBeanMBean);
                        mbeansToRegister.put(oMBean, onMessageDrivenBean);
                        // Update the EJBModule
                        ejbModuleMBean.addEjb(onMessageDrivenBean.toString());
                    }
                } catch (Exception e) {
                    servlog.log(BasicLevel.ERROR, "Cannot create container: " + e);
                }
            }
        }
        // Deployment was successful, add the bean as deployed and create the MBeans
        containers.addElement(cont);
        for (Iterator it = mbeansToRegister.keySet().iterator(); it.hasNext();) {
            ModelMBean mbean = (ModelMBean) it.next();
            ObjectName oName = (ObjectName) mbeansToRegister.get(mbean);
            mbeanServer.registerMBean(mbean, oName);

        }

        servlog.log(BasicLevel.DEBUG, "created container for " + fileName);
        //return cont;
        return onEjbModule.toString();
    }

    /**
     * Get the Container by its file name (.xml or .jar) AD. Keep only one
     * Enumeration of containers
     * @param fileName given file name on which retrieve the enumeration
     * @return enumeration of containers
     */
    public Container getContainer(String fileName) {
        servlog.log(BasicLevel.DEBUG, "entering for ejbjar= " + fileName);

        // In case of the name is a ejb-jar file name, check also in
        // the JONAS_BASE/ejbjars directory and JONAS_BASE/ejbjars/autoload

        String ejbjarFileName = EJBJARS_DIR + File.separator + fileName;
        File f = null;
        String pathEjbJarDirFile = "";
        String pathFile = "";
        try {
            f = new File(ejbjarFileName).getCanonicalFile();
            // Force pathEjbJarDirFile with slash to avoid conflict under
            // Windows (slash and back-slash)
            pathEjbJarDirFile = f.toURL().getPath();
        } catch (Exception e) {
            ; // If filename was an absolute path, it can throw an exception
            // The pathFile will be empty
        }
        try {
            f = new File(fileName).getCanonicalFile();
            // Force pathFile with slash to avoid conflict under Windows (slash
            // and back-slash)
            pathFile = f.toURL().getPath();
        } catch (Exception e) {
            String err = "Error while trying to get canonical file '" + fileName + "'";
            servlog.log(BasicLevel.ERROR, err);
            return null;
        }

        // for each container loaded
        Enumeration lc = containers.elements();
        while (lc.hasMoreElements()) {
            Container cont = (JContainer) lc.nextElement();
            String contName = cont.getExternalFileName();
            servlog.log(BasicLevel.DEBUG, "contName=" + contName);

            // first, search abs path
            servlog.log(BasicLevel.DEBUG, "pathFile= " + pathFile);
            servlog.log(BasicLevel.DEBUG, "pathEjbJarDirFile= " + pathEjbJarDirFile);
            if (contName.equals(pathFile)) {
                return cont;
            } else if (contName.equals(pathEjbJarDirFile)) {
                // next, search in ejbjar path
                return cont;
            } else {
                // else search in autoloadir
                List lAlDirs = getAutoloadDirectories();
                for (int i = 0; i < lAlDirs.size(); i++) {
                    String alDir = (String) lAlDirs.get(i);
                    String alFileName = alDir + File.separator + fileName;

                    try {
                        f = new File(alFileName).getCanonicalFile();
                        // Force alFileName with slash to avoid conflict under
                        // Windows (slash and back-slash)
                        alFileName = f.toURL().getPath();
                    } catch (Exception e) {
                        ; // If filename was an absolute path, it can throw an
                          // exception
                    }
                    servlog.log(BasicLevel.DEBUG, "alFileName= " + alFileName);

                    if (contName.equals(alFileName)) {
                        return cont;
                    }
                }
            }
            // else, go to the next loop
        }

        // not found
        return null;
    }

    /**
     * Remove a JOnAS container
     */
    public void removeContainer(Container cont) {
        servlog.log(BasicLevel.DEBUG, "entering for " + cont.getName());

        if (mbeanServer != null) {
            String moduleName = cont.getName();
            String earFileName = cont.getEarFileName();
            String earName = null;
            if (earFileName != null) {
                earName = EarServiceImpl.buildJ2eeApplicationName(earFileName);
            }
            String domainName = getDomainName();
            String serverName = getJonasServerName();
            ObjectName onEjbModule = J2eeObjectName.getEJBModule(domainName, serverName, earName, moduleName);

            // try to remove MBeans corresponding to the EJBs in the module
            ObjectName onEjb = null;
            try {
                String[] s_onEjbs = (String[]) mbeanServer.getAttribute(onEjbModule, "ejbs");
                for (int i = 0; i < s_onEjbs.length; i++) {
                    onEjb = new ObjectName(s_onEjbs[i]);
                    mbeanServer.unregisterMBean(onEjb);
                }
            } catch (Exception e) {
                servlog.log(BasicLevel.ERROR, "Problem when trying to unregsiter MBean " + onEjb.toString() + ": " + e);
            }

            // try to remove MBean corresponding to the module
            try {
                mbeanServer.unregisterMBean(onEjbModule);
                if (servlog.isLoggable(BasicLevel.DEBUG)) {
                    servlog.log(BasicLevel.DEBUG, "unregistered MBean: " + onEjbModule.toString());
                }
            } catch (Exception e) {
                servlog.log(BasicLevel.ERROR, "Problem when trying to unregsiter MBean " + onEjbModule.toString()
                        + ": " + e);
            }
        }
        cont.remove();
        containers.removeElement(cont);

        // Run the garbage collector
        Runtime.getRuntime().gc();

    }

    /**
     * List JOnAS container created by EJB Service
     */
    public Container[] listContainers() {
        servlog.log(BasicLevel.DEBUG, "entering for size= " + containers.size());

        Container[] ret = new Container[containers.size()];
        containers.copyInto(ret);
        return ret;
    }

    /**
     * Synchronized all entity bean containers
     * @param passivate passivate instances after synchronization.
     */
    public void syncAllEntities(boolean passivate) {
        servlog.log(BasicLevel.DEBUG, "");

        Enumeration lc = containers.elements();
        while (lc.hasMoreElements()) {
            JContainer cont = (JContainer) lc.nextElement();
            cont.syncAll(passivate);
        }
    }

    /** ******* MBEan methods ******** */

    /**
     * MBean method
     * @return Integer Total Number of Container currently in JOnAS
     */
    public Integer getCurrentNumberOfContainer() {
        return new Integer(containers.size());
    }

    /**
     * MBean method
     * @return Integer Total Number of Bean Type currently in JOnAS
     */
    public Integer getTotalCurrentNumberOfBeanType() {
        int count = 0;
        Enumeration lc = containers.elements();
        while (lc.hasMoreElements()) {
            JContainer cont = (JContainer) lc.nextElement();
            count += cont.getBeanNb();
        }
        return new Integer(count);
    }

    /**
     * MBean method
     * @return Integer Total Number of Bmp Type currently in JOnAS
     */
    public Integer getTotalCurrentNumberOfBMPType() {
        int count = 0;
        Enumeration lc = containers.elements();
        while (lc.hasMoreElements()) {
            JContainer cont = (JContainer) lc.nextElement();
            count += cont.getEntityBMPNb();
        }
        return new Integer(count);
    }

    /**
     * MBean method
     * @return Integer Total Number of Cmp Type currently in JOnAS
     */
    public Integer getTotalCurrentNumberOfCMPType() {
        int count = 0;
        Enumeration lc = containers.elements();
        while (lc.hasMoreElements()) {
            JContainer cont = (JContainer) lc.nextElement();
            count += cont.getEntityCMPNb();
        }
        return new Integer(count);
    }

    /**
     * MBean method
     * @return Integer Total Number of Sbf Type currently in JOnAS
     */
    public Integer getTotalCurrentNumberOfSBFType() {
        int count = 0;
        Enumeration lc = containers.elements();
        while (lc.hasMoreElements()) {
            JContainer cont = (JContainer) lc.nextElement();
            count += cont.getStatefulSessionNb();
        }
        return new Integer(count);
    }

    /**
     * MBean method
     * @return Integer Total Number of Sbl Type currently in JOnAS
     */
    public Integer getTotalCurrentNumberOfSBLType() {
        int count = 0;
        Enumeration lc = containers.elements();
        while (lc.hasMoreElements()) {
            JContainer cont = (JContainer) lc.nextElement();
            count += cont.getStatelessSessionNb();
        }
        return new Integer(count);
    }

    /**
     * MBean method
     * @return Integer Total Number of Mdb Type currently in JOnAS
     */
    public Integer getTotalCurrentNumberOfMDBType() {
        int count = 0;
        Enumeration lc = containers.elements();
        while (lc.hasMoreElements()) {
            JContainer cont = (JContainer) lc.nextElement();
            count += cont.getMessageDrivenNb();
        }
        return new Integer(count);
    }

    public void removeContainer(String fileName) throws Exception {
        removeContainerMBean(fileName);
    }

    /**
     * MBean method remove a container application
     * @param fileName the Name of this container
     */
    public void removeContainerMBean(String fileName) throws Exception {
        Container cont = null;
        try {
            cont = getContainer(fileName);
        } catch (Exception e) {
            String err = "Error while trying to find file '" + fileName + "'";
            servlog.log(BasicLevel.ERROR, err);
            throw new Exception(err);
        }

        if (cont != null) {
            removeContainer(cont, false);
        } else {
            String err = "Cannot remove the non-existant container '" + fileName + "'";
            servlog.log(BasicLevel.ERROR, err);
            throw new Exception(err);
        }
    }

    public String createContainer(String file) throws Exception {
        return createContainerMBean(file);
    }

    /**
     * MBean method allowing to create an EJB container for the EJBs contained
     * in a .jar (or .xml) file.
     * @param file Name of the file to be deployed
     * @return The ObjectName of the MBean associated to the container (to the
     *         deployed module)
     */
    public String createContainerMBean(String file) throws Exception {
        String ejbModule_on = null;
        try {
            Context contctx = new CompNamingContext(file);
            contctx.rebind("filename", file);
            ejbModule_on = createContainer(contctx);
        } catch (Exception e) {
            throw new Exception("Cannot create Container: " + e.toString());
        }
        return ejbModule_on;
    }

    /**
     * Test if the specified file is already deployed (if a container is created
     * for this jar).
     * @param fileName the name of the jar file
     * @return true if the jar was deployed, false otherwise
     */
    public Boolean isJarDeployed(String fileName) {
        return new Boolean(isJarLoaded(fileName));
    }

    // Kind of resources:
    static final int DATASOURCE = 1;

    static final int JMS_DESTINATION = 2;

    static final int JMS_FACTORY = 3;

    static final int MAIL_FACTORY = 4;

    /**
     * This method is used by the getXXXDependence MBean methods. It gathers the
     * ObjectNames of the EJBs using a given resource.
     * @param name the resource name
     * @type the resource type
     * @return the ObjectNames of all the ejb using this resource
     */
    Set getDependence(String name, int type) {
        Set resultObjectName = null; // to be returned by this method
        Set resultProperties = new HashSet(); // constructed by the JContainer
        Enumeration lc = containers.elements();
        try {
            Context ctx = new InitialContext();
            ctx.lookup(name);
            ctx.close();
            // this resource is rebind
            while (lc.hasMoreElements()) {
                JContainer cont = (JContainer) lc.nextElement();
                Set depProps = null; // Dependences described as a set of
                                     // Properties objects
                switch (type) {

                    case DATASOURCE:
                        depProps = cont.getDataSourceDependence(name);
                        break;

                    case JMS_DESTINATION:
                        depProps = cont.getJmsDestinationDependence(name);
                        break;

                    case JMS_FACTORY:
                        depProps = cont.getJmsConnectionFactoryDependence(name);
                        break;

                    case MAIL_FACTORY:
                        depProps = cont.getMailFactoryDependence(name);
                        break;

                }
                resultProperties.addAll(depProps);
                // change string to ObjectName
                try {
                    resultObjectName = convertToObjectNames(resultProperties);
                } catch (Exception e) {
                    servlog.log(BasicLevel.ERROR, "EjbServiceImpl: Object Name Error:" + e);
                }
            }
        } catch (NamingException ne) {
            // unexisting ds
            resultObjectName = new HashSet();
        }
        return resultObjectName;
    }

    Set convertToObjectNames(Set resultProperties) {
        Set resultObjectName = new HashSet();
        Iterator it = resultProperties.iterator();
        String domainName = getDomainName();
        String serverName = getJonasServerName();
        Properties item = null; // a item in resultProperties
        String ejbType = null;
        String ejbName = null;
        String fileName = null;
        String earFileName = null;
        String moduleName = null;
        String j2eeAppName = null;
        while (it.hasNext()) {
            item = (Properties) it.next();
            ObjectName ejbObjectName = null;
            // See JContainer.beansDependence() method for info about the
            // Properties object containt
            ejbType = item.getProperty("type");
            ejbName = item.getProperty("name");
            fileName = item.getProperty("fname");
            earFileName = item.getProperty("earFileName");
            moduleName = buildEJBModuleName(fileName);
            if (earFileName != null) {
                j2eeAppName = EarServiceImpl.buildJ2eeApplicationName(earFileName);
            }

            if (ejbType.equals("ejbbmp") || ejbType.equals("ejbcmp")) {
                ejbObjectName = J2eeObjectName.getEntityBean(domainName, moduleName, serverName, j2eeAppName, ejbName);
            } else if (ejbType.equals("ejbsbf")) {
                ejbObjectName = J2eeObjectName.getStatefulSessionBean(domainName, moduleName, serverName, j2eeAppName,
                        ejbName);
            } else if (ejbType.equals("ejbsbl")) {
                ejbObjectName = J2eeObjectName.getStatelessSessionBean(domainName, moduleName, serverName, j2eeAppName,
                        ejbName);
            } else if (ejbType.equals("ejbmdb")) {
                ejbObjectName = J2eeObjectName.getMessageDrivenBean(domainName, moduleName, serverName, j2eeAppName,
                        ejbName);
            }
            if (ejbObjectName != null) {
                resultObjectName.add(ejbObjectName);
            }
        }
        return resultObjectName;
    }

    /**
     * MBean method
     * @return the ObjectName of all the ejbs using this datasource.
     */
    public Set getDataSourceDependence(String dsName) {
        return getDependence(dsName, DATASOURCE);
    }

    /**
     * Manangement method
     * @return the ObjectName of all the ejb using this destination.
     */
    public Set getJmsDestinationDependence(String destName) {
        return getDependence(destName, JMS_DESTINATION);
    }

    /**
     * Management method
     * @return the ObjectName of all the ejb using this Connection Factory.
     */
    public Set getJmsConnectionFactoryDependence(String cfName) {
        return getDependence(cfName, JMS_FACTORY);
    }

    /**
     * Management method
     * @return the ObjectName of all the ejb using a given Mail Factory.
     */
    public Set getMailFactoryDependence(String mfName) {
        return getDependence(mfName, MAIL_FACTORY);
    }

    /**
     * Return the list of installed EJB containers. The JAR files or the
     * directories with expanded EJB container are searched in
     * JONAS_BASE/ejbjars and all ejbjar directories 'autoload'.
     * @return The list of JAR files or the directories with expanded EJB
     *         container found
     * @throws Exception if the list can't be retrieved
     */
    public List getInstalledJars() throws Exception {
        // get JAR files found in JONAS_BASE/ejbjars
        ArrayList al = JModule.getInstalledContainersInDir(EJBJARS_DIR, JModule.EJBJAR_EXTENSION,
                JModule.EJBJAR_CHILD_DIR, JModule.EJBJAR_CONFIRM_FILE);
        // get JAR files found in all autoload directories
        for (int i = 0; i < autoloadDirectories.size(); i++) {
            al.addAll(JModule.getInstalledContainersInDir(autoloadDirectories.get(i).toString(),
                    JModule.EJBJAR_EXTENSION, JModule.EJBJAR_CHILD_DIR, JModule.EJBJAR_CONFIRM_FILE));
        }
        return al;
    }

    /**
     * Deploy the given ejb-jars of an ear file with the specified parent
     * classloader (ear classloader). (This method is only used for the ear
     * applications, not for the ejb-jar applications).
     * @param ctx the context containing the configuration to deploy the
     *        ejbjars. <BR>This context contains the following parameters :
     *        <BR>- earRootUrl the root of the ear application. <BR>- earURL
     *        filename of the EAR. <BR>- earClassLoader the ear classLoader of
     *        the ear application. <BR>- ejbClassLoader the ejb classLoader for
     *        the ejbjars. <BR>- jarURLs the list of the urls of the ejb-jars
     *        to deploy. <BR>- roleNames the role names of the security-role.
     *        <BR>
     * @throws ServiceException if an error occurs during the deployment.
     */
    public void deployJars(Context ctx) throws ServiceException {

        // Get the 6 parameters from the context
        // - earRootUrl the root of the ear application.
        // - earClassLoader the ear classLoader of the ear application.
        // - ejbClassLoader the ejb classLoader for the ejbjars.
        // - jarURLs the list of the urls of the ejb-jars to deploy.
        // - roleNames the role names of the security-role
        URL earRootUrl = null;
        URL earUrl = null;
        ClassLoader earClassLoader = null;
        ClassLoader ejbClassLoader = null;
        URL[] jarURLs = null;
        String[] roleNames = null;
        try {
            earRootUrl = (URL) ctx.lookup("earRootUrl");
            earUrl = (URL) ctx.lookup("earUrl");
            earClassLoader = (ClassLoader) ctx.lookup("earClassLoader");
            ejbClassLoader = (ClassLoader) ctx.lookup("ejbClassLoader");
            jarURLs = (URL[]) ctx.lookup("jarURLs");
            roleNames = (String[]) ctx.lookup("roleNames");
        } catch (NamingException e) {
            String err = "Error while getting parameter from context param :" + e.getMessage();
            servlog.log(BasicLevel.ERROR, err);
            throw new ServiceException(err, e);
        }

        // Deploy all the ejb-jars of the ear application.
        for (int i = 0; i < jarURLs.length; i++) {

            // Get the name of an ejb-jar to deploy.
            String fileName = jarURLs[i].getFile();
            servlog.log(BasicLevel.DEBUG, "Deploy '" + fileName + "' for the ear service.");

            // The context to give for the creation of the container
            // associated to the ejb-jar.
            Context contctx = null;
            try {
                contctx = new CompNamingContext(fileName);
                contctx.rebind("filename", fileName);
                contctx.rebind("earClassLoader", earClassLoader);
                contctx.rebind("ejbClassLoader", ejbClassLoader);
                contctx.rebind("earRoot", earRootUrl.getFile());
                contctx.rebind("earFileName", earUrl.getFile());
                contctx.rebind("roleNames", roleNames);
                createContainer(contctx);
            } catch (Exception e) {
                // An ejb-jar is corrupted so undeploy all the deployed jar
                // of the ear application.
                servlog.log(BasicLevel.ERROR, "Error when deploying '" + fileName + "'");
                servlog.log(BasicLevel.ERROR, e.getMessage());
                servlog.log(BasicLevel.ERROR, "Undeploy ejb-jar of the ear application");

                for (int j = 0; j <= i; j++) {
                    Container cont = getContainer(jarURLs[j].getFile());
                    if (cont != null) {
                        removeContainer(cont, true);
                    } else {
                        servlog.log(BasicLevel.ERROR, "Cannot remove the non-existant container '" + fileName + "'");
                    }
                }
                throw new ServiceException("Error during the deployment", e);
            }
        }
        //return ejbClassLoader;
    }

    /**
     * Undeploy the given ejb-jars of an ear file. (This method is only used for
     * the ear applications, not for the ejb-jar applications).
     * @param urls the list of the urls of the ejb-jars to undeploy.
     */
    public void unDeployJars(URL[] urls) {
        for (int i = 0; i < urls.length; i++) {
            String fileName = urls[i].getFile();
            Container cont = getContainer(urls[i].getFile());
            if (cont != null) {
                removeContainer(cont, true);
            } else {
                servlog.log(BasicLevel.ERROR, "Cannot remove the non-existant container '" + fileName + "'");
            }
        }
    }

    /**
     * Remove the specified container.
     * @param cont the container to remove.
     * @param isEarCase true if only if the removeContainer method is called in
     *        the ear case, false otherwise.
     */
    public void removeContainer(Container cont, boolean isEarCase) {
        if (isEarCase == (cont.getEarFileName() != null)) {
            removeContainer(cont);
        } else {
            String err = "Cannot remove container '" + cont.getName()
                    + "' it is in an ear application. You must undeploy the ear associated.";
            servlog.log(BasicLevel.ERROR, err);
        }
    }

    /**
     * Make a cleanup of the cache of deployment descriptor. This method must be
     * invoked after the ear deployment by the EAR service. the deployment of an
     * ear by .
     * @param earClassLoader the ClassLoader of the ear application to remove
     *        from the cache.
     */
    public void removeCache(ClassLoader earClassLoader) {
        EjbManagerWrapper.removeCache(earClassLoader);
    }

    /**
     * Add recursively the ejbjars of the specified directory. If the dir has a
     * relative path, this path is relative from where the Application Server is
     * launched. If the dir is not found it will be searched in
     * $JONAS_BASE/ejbjars/ directory.
     * @param dirPath the path to the directory containing the ejbjars to load.
     */
    private void addEjbjars(String dirPath) {
        boolean found = false;

        // Look the directory relative to where the JOnAS server is launch or
        // if the path is absolute.
        File dir = new File(dirPath);
        found = dir.isDirectory();

        if (!found) {
            // Look the directory relative to the $JONAS_BASE/ejbjars directory
            dir = new File(EJBJARS_DIR + File.separator + dirPath);
            found = dir.isDirectory();
        }

        if (found) {
            addEjbjarsFrom(dir);
        } else {
            String err = "Warning: Cannot load dir: '" + dirPath + "' ";
            err += "is not a directory or directory doesn't exist";
            servlog.log(BasicLevel.WARN, err);
        }
    }

    /**
     * Add the ejbjars of the specified directory.
     * @param dir the directory from which the ejbjars are loaded.
     * @throws JWebContainerServiceException if the argument is not a directory
     */
    private void addEjbjarsFrom(File dir) throws ServiceException {

        try {
            if (dir.isDirectory()) {
                File[] files = dir.listFiles();
                for (int i = 0; i < files.length; i++) {
                    if (files[i].isFile()) {
                        if (files[i].getPath().toLowerCase().endsWith(".jar")) {
                            ejbNames.add(files[i].getCanonicalPath());
                        }
                    } else {
                        addEjbjarsFrom(files[i]);
                    }
                }
            } else {
                String err = "Cannot load dir: '" + dir.getPath();
                err += "' is not a directory";
                servlog.log(BasicLevel.ERROR, err);
                throw new ServiceException(err);
            }
        } catch (IOException e) {
            String err = "Invalid file name '" + dir.getPath();
            servlog.log(BasicLevel.ERROR, err);
            throw new ServiceException(err, e);
        }
    }

    /**
     * Test if the specified filename is already deployed or not
     * @param fileName the name of the jar file.
     * @return true if the jar is deployed, else false.
     */
    public boolean isJarLoaded(String fileName) {
        return (getContainer(fileName) != null);
    }

    /**
     * Return the list of all loaded EJB container.
     * @return The list of deployed EJB container
     */
    public List getDeployedJars() {
        ArrayList al = new ArrayList();
        Container oContainer;
        for (Enumeration jars = containers.elements(); jars.hasMoreElements();) {
            oContainer = (Container) jars.nextElement();
            try {
                al.add((new File(oContainer.getExternalFileName())).toURL().getPath());
            } catch (Exception e) {
                // none
            }
        }
        return al;
    }

    /**
     * Return the list of installed EJB container ready to deploy.
     * @return The list of deployable EJB container
     * @throws Exception
     */
    public List getDeployableJars() throws Exception {
        List al = getInstalledJars();
        al.removeAll(getDeployedJars());
        return al;
    }

    /**
     * Return the list of "autoload" directories for EJB containers.
     * @return The list of all "autoload" directories
     */
    public List getAutoloadDirectories() {
        ArrayList al = new ArrayList();
        for (int i = 0; i < autoloadDirectories.size(); i++) {
            try {
                al.add((new File(autoloadDirectories.get(i).toString())).toURL().getPath());
            } catch (Exception e) {
                // none
            }
        }
        return al;
    }

    /**
     * Return the Ejbjars directory.
     * @return The Ejbjars directory
     */
    public String getEjbjarsDirectory() {
        String sRet = null;
        try {
            sRet = (new File(EJBJARS_DIR)).toURL().getPath();
        } catch (Exception e) {
            // none
        }
        return sRet;
    }

    /**
     * Build the EJBModule name.
     * @param p_Url The URL of Jar file
     * @return The EJBModule name
     */
    public static String buildEJBModuleName(URL p_Url) {
        String sName = null;
        try {
            sName = new File(p_Url.getFile()).getName();
            if ("file".equals(p_Url.getProtocol())) {
                sName = buildEJBModuleName(sName);
            }
        } catch (NullPointerException e) {
            // none action
        }
        return sName;
    }

    /**
     * Build the EJBModule name.
     * @param p_Filename The name of Jar file
     * @return The EJBModule name
     */
    public static String buildEJBModuleName(String p_Filename) {
        String sName = null;
        try {
            sName = new File(p_Filename).getName();
            int iPos = sName.lastIndexOf('.');
            if (iPos > -1) {
                sName = sName.substring(0, iPos);
            }
        } catch (NullPointerException e) {
            // none action
        }
        return sName;
    }
}