/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Rar.java,v 1.23 2004/06/25 13:35:48 pelletib Exp $
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.resource;

import java.io.File;
import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;
import java.util.jar.JarEntry;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.modelmbean.ModelMBean;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.StringRefAddr;
import javax.resource.Referenceable;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ManagedConnectionFactory;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.ResourceAdapterAssociation;
import javax.resource.spi.ResourceAllocationException;

import org.apache.commons.modeler.ManagedBean;
import org.apache.commons.modeler.Registry;
import org.objectweb.jonas.common.JJarFile;
import org.objectweb.jonas.common.JProp;
import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.ear.EarServiceImpl;
import org.objectweb.jonas.jmx.J2eeObjectName;
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.jtm.TransactionService;
import org.objectweb.jonas.management.JonasMBeanTools;
import org.objectweb.jonas.naming.CompNamingContext;
import org.objectweb.jonas.server.J2EEServerMBean;
import org.objectweb.jonas.server.JClassLoader;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.service.ServiceManager;
import org.objectweb.jonas_ejb.deployment.xml.ActivationConfigProperty;
import org.objectweb.jonas_lib.deployment.xml.JLinkedList;
import org.objectweb.jonas_rar.deployment.api.RarDeploymentDesc;
import org.objectweb.jonas_rar.deployment.lib.wrapper.RarManagerWrapper;
import org.objectweb.jonas_rar.deployment.xml.Adminobject;
import org.objectweb.jonas_rar.deployment.xml.AuthenticationMechanism;
import org.objectweb.jonas_rar.deployment.xml.ConfigProperty;
import org.objectweb.jonas_rar.deployment.xml.ConnectionDefinition;
import org.objectweb.jonas_rar.deployment.xml.Connector;
import org.objectweb.jonas_rar.deployment.xml.InboundResourceadapter;
import org.objectweb.jonas_rar.deployment.xml.JdbcConnParams;
import org.objectweb.jonas_rar.deployment.xml.JonasActivationspec;
import org.objectweb.jonas_rar.deployment.xml.JonasAdminobject;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigProperty;
import org.objectweb.jonas_rar.deployment.xml.JonasConnectionDefinition;
import org.objectweb.jonas_rar.deployment.xml.JonasConnector;
import org.objectweb.jonas_rar.deployment.xml.Messageadapter;
import org.objectweb.jonas_rar.deployment.xml.Messagelistener;
import org.objectweb.jonas_rar.deployment.xml.OutboundResourceadapter;
import org.objectweb.jonas_rar.deployment.xml.PoolParams;
import org.objectweb.jonas_rar.deployment.xml.RequiredConfigProperty;
import org.objectweb.jonas_rar.deployment.xml.Resourceadapter;
import org.objectweb.transaction.jta.TransactionManager;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Rar object
 * @author Eric Hardesty
 * Contributor(s):
 *
 */

public class Rar {

    // Loggers used in the ResourceService
    private static Logger logger = null;
    private static Logger poolLogger = null;
    private static Logger setterLogger = null;
    private static Logger manageLogger = null;

    String rarFileName = null;
    JClassLoader curLoader = null;
    boolean isInEar;
    URL earUrl = null;
    Context rCtx = null;
    String jDomain = null;
    String jServer = null;
    Vector jndinames = new Vector();

    JCAResource jcaResourceMBean = null;
    String JCAResourceName = null;
    Registry oRegistry = null;

    // J2EE CA 1.5 objects
    private ResourceWorkManager workMgr = null;
    private ResourceBootstrapContext bootCtx = null;

    // Properties for inits

    // JCA resource service configuration parameters
    public static final String CLASS = "jonas.service.resource.class";
    public static final String JNDI_NAME = "jndiname";
    public static final String OBJ_TYPE = "objtype";
    public static final String RESOURCE_LIST = "jonas.service.resource.resources";

    public final String MANAGED_CONNECTION_FACTORY_CLASS = "managedconnectionfactoryclass";

    public static final int DEF_WRK_THREADS = 5;
    public static final int DEF_EXEC_TIME = 0;

    public static final String JCD = "JCD";
    public static final String JAS = "JAS";
    public static final String JAO = "JAO";

    public String objectName = null;
    public String pathName = null;
    public ResourceAdapter resAdp = null;
    public Connector raConn = null;
    public JonasConnector lnkJonasConn = null;
    public JonasConnector jonasConn = null;

    public String xmlContent = null;
    public String jonasXmlContent = null;


    private class ConfigObj {
        public String type;
        public int offset;
        public String jndiName;
        public String interfaceStr;
        public String classStr;
        public Object factory;
        public ConnectionManager cm;
        public boolean basicPass;
        public JLinkedList reqConfigProps;
        public ConfigObj(String fType, int off, String jndi,
                          String intStr, String clsStr, Object fact) {
            type = fType;
            offset = off;
            jndiName = jndi;
            interfaceStr = intStr;
            classStr = clsStr;
            factory = fact;
            cm = null;
            reqConfigProps = null;
        }
    }

    private class intPoolParams {
        int poolMax = -1;
        int poolMin = 0;
        int poolInit = 0;
        int pstmtMax = 0;
        long maxage = 0;
        int jdbcConnLevel = 0;
        String jdbcConnTestStmt = "";
        public intPoolParams() { }
    }

    /**
     * These hashtables associate an RAR with other objects
     */
    //private Hashtable jndiName2RA = new Hashtable();
    private Hashtable cfgObjs = new Hashtable();

    /**
     * The transaction manager in the server
     */
    private TransactionManager tm = null ;

    /**
     * Reference to a MBean server.
     */
    private MBeanServer mbeanServer = null;

    /**
     * ObjectName for the jcaResource
     */
    private String jcaResourceObjectName = null;

    /**
     * The ConnectionManager manages connections of resource adapters
     * and more exactly the association between logical connections,
     * physical connections, local transactions and global transaction(XA)
     */
    private ConnectionManagerImpl cm = null;

    /**
     * JNDI Context
     */
    private Context ictx = null;


    /**
     *  The name of the JONAS_BASE directory
     */
    private static final String JONAS_BASE = JProp.getJonasBase();
    /**
     * The name of the working directory.
     */
    protected static final String WORK_DIR =  JProp.getWorkDir();

    /**
     * The name of the working apps directory.
     */
    protected static final String WORK_RARS_DIR = WORK_DIR + File.separator + "rars";

    /**
     * - Get the loggers
     * - Get the global jndi context
     * - Get the list of the resource adapters. The list is reachable in the
     * -  context parameter under the name RESOURCE_LIST.
     * - Get the transaction manager into the jndi
     * - Set the XML validation property
     */
    public Rar(Context ctx, String jDom, String jServ,
                ResourceWorkManager wrkMgr, ResourceBootstrapContext btCtx) {
        if(logger == null)
            logger = Log.getLogger(Log.JONAS_JCA_PREFIX+".process");
        if(poolLogger == null)
            poolLogger = Log.getLogger(Log.JONAS_JCA_PREFIX+".pool");
        if(setterLogger == null)
            setterLogger = Log.getLogger(Log.JONAS_JCA_PREFIX+".setters");
        if(manageLogger == null)
            manageLogger = Log.getLogger(Log.JONAS_JCA_PREFIX+".management");

        try {
            rarFileName = (String) ctx.lookup("rarFileName");
            isInEar = ((Boolean) ctx.lookup("isInEar")).booleanValue();
            if (isInEar == true) {
                earUrl = (URL) ctx.lookup("earUrl");
            }
            curLoader = (JClassLoader) ctx.lookup("classloader");

        } catch (NamingException e) {
            String err = "Error while getting parameter from context param.";
            logger.log(BasicLevel.ERROR, err + e.getMessage());
            throw new ResourceServiceException(err, e);
        } catch (Exception ex) {
            String err = "Error while getting parameter from context param.";
            logger.log(BasicLevel.ERROR, err + ex.getMessage());
            throw new ResourceServiceException(err, ex);
        }

        workMgr = wrkMgr;
        bootCtx = btCtx;
        rCtx = ctx;
        jDomain = jDom;
        jServer = jServ;

        // Avoid using NamingManager here: performance is not a goal here.
        try {
            ictx = new InitialContext();
        } catch (NamingException e) {
            logger.log(BasicLevel.ERROR,"Cannot create initial context when Resource service initializing");
            throw new ServiceException("Cannot create initial context when Resource service initializing",e);
        }

        // Get a reference to the Transaction service
        try {
            ServiceManager sm = ServiceManager.getInstance();
            TransactionService ts = (TransactionService) sm.getTransactionService();
            tm = ts.getTransactionManager();
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR,"Cannot get the Transaction service: "+e);
            throw new ServiceException("Cannot get the Transaction service: ", e);
        }

        // Get the JMX Server via JMX Service
        try {
            mbeanServer =
                ((JmxService) ServiceManager.getInstance().getJmxService()).getJmxServer();
        } catch (Exception e) {
            // the JMX service may not be started
            mbeanServer = null;
        }

        // Use Jakarta Common Modeler API
        oRegistry = JonasMBeanTools.getRegistry();

    }

    /*
     * Process the resource adapter.  This Resource Adapter is configured via
     * xml files in the rar file
     */
    public Context processRar() throws Exception {

        String onRar = null;
        // Get the ra.xml and jonas-ra.xml descriptions
        RarDeploymentDesc radesc = RarManagerWrapper.getInstance(rCtx);
        Connector conn = radesc.getConnector();
        JonasConnector jonasConn = radesc.getJonasConnector();
        xmlContent = radesc.getXmlContent();
        jonasXmlContent = radesc.getJOnASXmlContent();
        if(conn == null && jonasConn == null){
            logger.log(BasicLevel.ERROR,"Rar.processRar: Resource ("+rarFileName+") must be a valid RAR file.");
            throw new Exception("resource input file incorrect");
        }

        // Need to check if rarlink is specified.  If yes and has been processed, then get the correct
        // Connector object and continue, if it has not been processed, then add this rar to the
        // deferred list and it will be processed at end of the rar files.  If no rarlink, then
        // make sure that the connector object is valid.
        String rarVal = null;
        if (jonasConn != null) {
            rarVal = (jonasConn.getRarlink() == null
                      ? null
                      : jonasConn.getRarlink());
            if (rarVal != null && rarVal.length() > 0) {
                conn = getConnector(rarVal);
                if (conn == null) {
                    return rCtx;
                }
                xmlContent = getXmlContent(rarVal);
                // Get the config parameters
                lnkJonasConn = getJonasConnector(rarVal);
            } else if (conn == null) {
                logger.log(BasicLevel.ERROR,"Rar.processRar: Resource ("+rarFileName+") is not valid.");
                throw new Exception("resource input file incorrect: no ra.xml file");
            }
            bldSecurityTable(lnkJonasConn, jonasConn);
            raConn = conn;
        }
        else {
            logger.log(BasicLevel.ERROR,"Rar.processRar: Resource ("+rarFileName+") must be a valid RAR file, there must be jonas-ra.xml file.");
            throw new Exception("resource input file incorrect: no jonas-ra.xml file");
        }

        // Setup the global pool parameters
        intPoolParams pParams = new intPoolParams();
        setPoolParams(jonasConn.getPoolParams(), jonasConn.getJdbcConnParams(), pParams);

        // Process the RAR file adding each of the jar files into the classloader and if the
        // native-lib is defined then place all other files into that directory.

        extractJars(rarFileName, jonasConn);

        // Process the config values
        Resourceadapter ra = conn.getResourceadapter();

        // Check Authentication Mechanisms, must if some exist then BasicPassword
        // must be present
        boolean basicPass = true;
        JLinkedList authList = ra.getAuthenticationMechanismList();
        if (authList != null && authList.size() > 0) {
            basicPass = false;
            for (Iterator i = authList.iterator(); i.hasNext(); ) {
                AuthenticationMechanism am = (AuthenticationMechanism) i.next();
                if (am.getAuthenticationMechanismType().equalsIgnoreCase("BasicPassword")) {
                    basicPass = true;
                    break;
                }
            }
            if (!basicPass) {
                logger.log(BasicLevel.ERROR,"Rar.processRar: Resource ("+rarFileName+") doesn't contain an AuthenticationMechanismType that is supported by JOnAS(BasicPassword).");
                throw new Exception("No AuthenticationMechanismType that is supported by JOnAS(BasicPassword).");
            }
        }

        ConfigProperty [] cfgRaJonas = null;

        String logEnabled = null;
        if (jonasConn.getLogEnabled() != null) {
            logEnabled = jonasConn.getLogEnabled().trim();
        }
        String logTopic = null;
        if (jonasConn.getLogTopic() != null) {
            logTopic = jonasConn.getLogTopic().trim();
        }

        if (conn.getSpecVersion().equals("1.0")) {

            if (lnkJonasConn != null) {
                cfgRaJonas = buildConfigProperty(ra.getConfigPropertyList(),
                                                 jonasConn.getJonasConfigPropertyList(),
                                                 lnkJonasConn.getJonasConfigPropertyList());
            } else {
                cfgRaJonas = buildConfigProperty(ra.getConfigPropertyList(),
                                                 jonasConn.getJonasConfigPropertyList(), null);
            }

            onRar = registerRarMBean(cfgRaJonas);
            processMCF(ra.getManagedconnectionfactoryClass(), ra.getTransactionSupport(),
                       jonasConn.getJndiName().trim(), logEnabled, logTopic, cfgRaJonas, pParams, basicPass, 0);

        } else if (conn.getSpecVersion().equals("1.5")) {
            String raStr = conn.getResourceadapter().getResourceadapterClass().trim();
            boolean isResAdapt = false;
            logger.log(BasicLevel.INFO, "Starting deployment of " + rarFileName);
            if (raStr != null && raStr.length() > 0){
                isResAdapt = true;

                // Instantiate the resource adapter class
                // Class raClass = Class.forName(raStr, true, curLoader);
                Class raClass = curLoader.loadClass(raStr);
                resAdp = (ResourceAdapter) raClass.newInstance();

                if (lnkJonasConn != null) {
                    cfgRaJonas = buildConfigProperty(ra.getConfigPropertyList(),
                                                     jonasConn.getJonasConfigPropertyList(),
                                                     lnkJonasConn.getJonasConfigPropertyList());
                } else {
                    cfgRaJonas = buildConfigProperty(ra.getConfigPropertyList(),
                                                     jonasConn.getJonasConfigPropertyList(), null);
                }
                processSetters(raClass, resAdp, rarFileName, cfgRaJonas);

                // Call Resourceadapter start method
                try {
                    resAdp.start(bootCtx);
                } catch (Exception ex) {
                    logger.log(BasicLevel.ERROR,"Rar: Error from resource ("+rarFileName+") start method.");
                    throw new Exception("Error from start method. " + ex);
                } catch (Throwable th) {
                    logger.log(BasicLevel.ERROR,"Rar: Error from resource ("+rarFileName+") start method.");
                    throw new Exception("Error from start method. ", th);
                }
            }

            onRar = registerRarMBean(cfgRaJonas);

            // Loop thru ConnectionDefinition(outbound) factories
            OutboundResourceadapter outRa = ra.getOutboundResourceadapter();
            JLinkedList cdList = (outRa == null ? null : outRa.getConnectionDefinitionList());
            ConnectionDefinition conDef = null;
            JonasConnectionDefinition jConDef = null;
            String id = null;
            int idOffset = -1;
            if (cdList != null) {
                for (int cd = 0; cd < cdList.size(); cd++) {
                    conDef = (ConnectionDefinition) cdList.get(cd);
                    id = conDef.getId();
                    idOffset++;
                    jConDef = (JonasConnectionDefinition) getJonasXML(jonasConn, id, idOffset, JCD);


                    if (jConDef.getLogEnabled() != null) {
                        logEnabled = jConDef.getLogEnabled().trim();
                    }
                    if (jConDef.getLogTopic() != null) {
                        logTopic = jConDef.getLogTopic().trim();
                    }
                    cfgRaJonas = buildConfigProperty(conDef.getConfigPropertyList(),
                                                     jConDef.getJonasConfigPropertyList(), null);

                    processMCF(conDef.getManagedconnectionfactoryClass(), outRa.getTransactionSupport(),
                               jConDef.getJndiName().trim(), logEnabled, logTopic, cfgRaJonas, pParams, basicPass, idOffset);
                }
            }

            // We only want to do anything with Inbound or Adminobjects if the Resourceadapter
            // class was specified, otherwise it is a 1.0 RAR with a 1.5 ra.xml
            if (isResAdapt) {
                // Loop thru Messagelisteners (inbound) factories
                InboundResourceadapter inAdapt = ra.getInboundResourceadapter();
                Messageadapter msgAdapt = null;
                JLinkedList mlList = null;
                if (inAdapt != null) {
                    msgAdapt = inAdapt.getMessageadapter();
                    mlList = (msgAdapt == null ? null : msgAdapt.getMessagelistenerList() );
                }
                Messagelistener msgList = null;
                JonasActivationspec jAct = null;
                id = null;
                idOffset = -1;
                if (mlList != null) {
                    for (int ml = 0; ml < mlList.size(); ml++) {
                        msgList = (Messagelistener) mlList.get(ml);
                        id = msgList.getId();
                        idOffset++;
                        jAct = (JonasActivationspec) getJonasXML(jonasConn, id, idOffset, JAS);

                        processJAS(msgList, jAct, idOffset);
                    }
                }

                // Loop thru Adminobjects
                JLinkedList aoList = ra.getAdminobjectList();
                Adminobject admObj = null;
                JonasAdminobject jAObj = null;
                id = null;
                idOffset = -1;
                if (aoList != null) {
                    for (int ao = 0; ao < aoList.size(); ao++) {
                        admObj = (Adminobject) aoList.get(ao);
                        id = admObj.getId();
                        idOffset++;
                        jAObj = (JonasAdminobject) getJonasXML(jonasConn, id, idOffset, JAO);

                        cfgRaJonas = buildConfigProperty(admObj.getConfigPropertyList(), jAObj.getJonasConfigPropertyList(), null);

                        processJAO(admObj, jAObj, cfgRaJonas, idOffset);
                    }
                }

                // Set initialized when everything is deployed
                workMgr.setInited();
            }
        } else {
            logger.log(BasicLevel.ERROR,"ResourceService.createRA: Resource ("+rarFileName+") must be specify a valid specification of 1.0 or 1.5.");
            throw new Exception("resource input file incorrect: invalid specification support(only 1.0 & 1.5 are valid)");
        }

        logger.log(BasicLevel.INFO, rarFileName+" available");
        try {
            rCtx.rebind("onRar", onRar);
            rCtx.rebind("deployed", new Boolean(true));
        } catch (Exception ex) {
            String err = "Error while getting parameter from context param.";
            logger.log(BasicLevel.ERROR, err + ex.getMessage());
            throw new ResourceServiceException(err, ex);
        }
        return rCtx;
    }



    /*
     * Register the Rar.
     */
    private String registerRarMBean(ConfigProperty [] cfgRaJonas) throws Exception {

        String jndiName = null;  // Just for now to get thru the compile

        // --------------------------
        // Register MBeans cf. JSR 77
        // --------------------------
        if (mbeanServer != null) {

            // ------------------------------------------------
            // Register this ResourceAdapter with JMX - old way
            // ------------------------------------------------
            String raName = null; // ResourceAdapter name
            Properties p = buildProp(cfgRaJonas);

            // MBean resourcesource registering
            String jmxRaName = (new File(rarFileName)).toURL().getFile().replace(':', '|');
            mbeanServer.registerMBean(new JmxResourceAdapter(p, jndiName, rarFileName, isInEar, earUrl),
                                      JonasObjectName.resourceAdapter(jmxRaName));

            // ResourceAdapterModule MBean
            // ---------------------------
            String appName = null;
            if (isInEar) {
                appName = EarServiceImpl.buildJ2eeApplicationName(earUrl);
            }

            // ResourceAdapter Module name
            String resourceAdapaterModuleName = buildModuleName(jmxRaName);
            ObjectName onResourceAdapterModule =
                J2eeObjectName.getResourceAdapterModule(jDomain,
                                                        jServer,
                                                        appName,
                                                        resourceAdapaterModuleName);

            if (manageLogger.isLoggable(BasicLevel.DEBUG)) {
                manageLogger.log(BasicLevel.DEBUG, "ObjectName created for ResourceAdapterModule: " + onResourceAdapterModule.toString());
            }

            ResourceAdapterModule raModuleMBean = new ResourceAdapterModule(onResourceAdapterModule.toString(),
                                                                            isInEar, earUrl);
            raModuleMBean.setFileName(jmxRaName);
            raModuleMBean.setDeploymentDescriptor(xmlContent);
            raModuleMBean.setJonasDeploymentDescriptor(jonasXmlContent);
            ManagedBean oManaged = oRegistry.findManagedBean("ResourceAdapterModule");

            ModelMBean oMBean = oManaged.createMBean(raModuleMBean);

            mbeanServer.registerMBean(oMBean, onResourceAdapterModule);
            if (manageLogger.isLoggable(BasicLevel.DEBUG)) {
                manageLogger.log(BasicLevel.DEBUG, "ResourceAdapterModule MBean created");
            }

            // ResourceAdapter MBean
            // ---------------------
            raName = "RA" + resourceAdapaterModuleName;
            ObjectName onResourceAdapter = J2eeObjectName.getResourceAdapter(jDomain,
                                                                             resourceAdapaterModuleName,
                                                                             appName,
                                                                             jServer,
                                                                             raName);
            org.objectweb.jonas.resource.ResourceAdapter raMBean =
                           new org.objectweb.jonas.resource.ResourceAdapter(onResourceAdapter.toString(),
                                                                            p,
                                                                            jndiName,
                                                                            jmxRaName,
                                                                            isInEar,
                                                                            earUrl);
            oManaged = oRegistry.findManagedBean("ResourceAdapter");
            oMBean = oManaged.createMBean(raMBean);
            if (manageLogger.isLoggable(BasicLevel.DEBUG)) {
                manageLogger.log(BasicLevel.DEBUG, "ResourceAdapter ManagedBean created");
            }
            mbeanServer.registerMBean(oMBean, onResourceAdapter);

            // Update the ResourceAdapterModule MBean with the ResourceAdapter MBean's OBJECT_NAME
            raModuleMBean.setResourceAdapter(onResourceAdapter.toString());
            if (manageLogger.isLoggable(BasicLevel.DEBUG)) {
                manageLogger.log(BasicLevel.DEBUG, "ResourceAdapterModule ManagedBean updated");
            }

            // JCA Resource implemented by this ResourceAdapter
            // ------------------------------------------------
            JCAResourceName = "JCAResource" + raName; // TO BE IMPLEMENTED
            ObjectName onJCAResource =
                J2eeObjectName.getJCAResource(jDomain,
                                              jServer,
                                              raName,
                                              JCAResourceName);
            jcaResourceMBean = new JCAResource(onJCAResource.toString());
            oManaged = oRegistry.findManagedBean("JCAResource");
            oMBean = oManaged.createMBean(jcaResourceMBean);
            if (manageLogger.isLoggable(BasicLevel.DEBUG)) {
                manageLogger.log(BasicLevel.DEBUG, "JCAResource J2EEResource created");
            }
            mbeanServer.registerMBean(oMBean, onJCAResource);

            // Update the ResourceAdapter MBean with the JCAResource MBean's OBJECT_NAME
            raMBean.setJcaResource(onJCAResource.toString());
            if (manageLogger.isLoggable(BasicLevel.DEBUG)) {
                manageLogger.log(BasicLevel.DEBUG, "JCAResource J2EEResource updated");
            }

            return onResourceAdapterModule.toString();
        }
        return null;
    }


    /*
     * Unregister the resource adapter.
     */
    public void unRegister() throws Exception {

        for (int i=0;i<jndinames.size();i++) {
            String jName = (String) jndinames.get(i);
            ConfigObj co = (ConfigObj) cfgObjs.get(jName);
            try {
                // Unbind from jndi
                ictx.unbind(jName);
            } catch (NamingException ex) {
                logger.log(BasicLevel.DEBUG,"ResourceService: cannot unbind jndiname of "+jName+" for ResourceAdapter "
                            +rarFileName+" :\n" + ex);
                throw new ServiceException(ex.getMessage());
            }

            if (co != null) {  // Add for now, take out for final
                if (co.cm != null) {
                    // The connection manger must stop managing the RA
                    ((ConnectionManagerImpl)co.cm).cleanResourceAdapter();
                }
            }

        }
        cfgObjs.clear();

        // --------------------------
        // unregister MBeans cf. JSR 77
        // --------------------------
        if (mbeanServer != null) {
            // MBean resourcesource registering
            String jmxRaName = (new File(rarFileName)).toURL().getFile().replace(':', '|');
            mbeanServer.unregisterMBean(JonasObjectName.resourceAdapter(jmxRaName));

            // Construct ResourceAdapterModule MBean name
            // ------------------------------------------
            String appName = null;
            if (isInEar) {
                appName = EarServiceImpl.buildJ2eeApplicationName(earUrl);
            }
            // ResourceAdapter Module name
            String resourceAdapaterModuleName = buildModuleName(jmxRaName);
            ObjectName onResourceAdapterModule =
                J2eeObjectName.getResourceAdapterModule(jDomain,
                                                    jServer,
                                                    appName,
                                                    resourceAdapaterModuleName);
            try {
                String[] resourceAdapaterNames = (String[]) mbeanServer.getAttribute(onResourceAdapterModule, "resourceAdapters");

                // Unregister ResourceAdapterModule MBean
                mbeanServer.unregisterMBean(onResourceAdapterModule);

                // Unregister ResourceAdapters
                for (int i=0 ; i<resourceAdapaterNames.length ; i++) {
                    String resourceAdapaterName = resourceAdapaterNames[i];
                    ObjectName onResourceAdapater = new ObjectName(resourceAdapaterName);
                    // Determine this ResourceAdapter's JCAResource
                    String jcaResourceName = (String) mbeanServer.getAttribute(onResourceAdapater, "jcaResource");
                    // Update the list of resources in the J2EEServer MBean with the JCAResource MBean's OBJECT_NAME
                    mbeanServer.unregisterMBean(onResourceAdapater);
                    ObjectName onJCAResource = new ObjectName(jcaResourceName);
                    String[] connectionFactoriesNames = (String[]) mbeanServer.getAttribute(onJCAResource, "connectionFactories");
                    // Unregister the JCA ConnectionFactories of this JCAResource
                    for (int j=0 ; j<connectionFactoriesNames.length ; j++) {
                        String connectionFactoryName = connectionFactoriesNames[j];
                        ObjectName onJCAConnectionFactory = new ObjectName(connectionFactoryName);
                        String managedConnectionFactoryName = (String) mbeanServer.getAttribute(onJCAConnectionFactory, "managedConnectionFactory");
                        mbeanServer.unregisterMBean(onJCAConnectionFactory);
                        mbeanServer.unregisterMBean(new ObjectName(managedConnectionFactoryName));
                    }
                    String[] adminObjectNames = (String[]) mbeanServer.getAttribute(onJCAResource, "adminObjects");
                    // Unregister the JCA AdminObjects of this JCAResource
                    for (int j=0 ; j<adminObjectNames.length ; j++) {
                        String adminObjectName = adminObjectNames[j];
                        ObjectName onJCAAdminObject = new ObjectName(adminObjectName);
                        mbeanServer.unregisterMBean(onJCAAdminObject);
                    }
                    String[] activationSpecNames = (String[]) mbeanServer.getAttribute(onJCAResource, "activationSpecs");
                    // Unregister the JCA ActivationSpecs of this JCAResource
                    for (int j=0 ; j<activationSpecNames.length ; j++) {
                        String activationSpecName = activationSpecNames[j];
                        ObjectName onJCAActivationSpec = new ObjectName(activationSpecName);
                        mbeanServer.unregisterMBean(onJCAActivationSpec);
                    }
                    mbeanServer.unregisterMBean(onJCAResource);
                }
            } catch (MalformedObjectNameException ma) {
                logger.log(BasicLevel.ERROR, "Cannot cleanly unregister RAR: "
                           + ma.getMessage());
                ma.printStackTrace();
            } catch (MBeanRegistrationException mr) {
                logger.log(BasicLevel.ERROR, "Cannot cleanly unregister RAR: "
                           + mr.getMessage());
                mr.printStackTrace();
            } catch (InstanceNotFoundException infe) {
                logger.log(BasicLevel.ERROR, "Cannot cleanly unregister RAR: "
                           + infe.getMessage());
                infe.printStackTrace();
            } catch (Exception e) {
               logger.log(BasicLevel.ERROR, "Cannot cleanly unregister RAR: "
                           + e.getMessage());
               e.printStackTrace();
            }
        }

        //   Call Resourceadapter stop
        if (resAdp != null) {
            resAdp.stop();
        }

    }

    //--------------------------------------------------------------
    //           Other Method
    //--------------------------------------------------------------

    /**
     * get the ResourceObject matching the jndiname specified
     * will be used in ResourceObjectJNDIHandler class at lookup time
     */
    public Object getResourceObject(String jndiName) {
        synchronized (cfgObjs) {
            ConfigObj co = (ConfigObj) cfgObjs.get(jndiName);
            return (co==null ? null : co.factory );
        }
    }

    /**
     * get the ResourceAdapter Connector object for this Rar
     */
    public Connector getConnector() {
        return raConn;
    }

    /**
     * get the Xml Content for this Rar
     */
    public String getXmlContent() {
        return xmlContent;
    }

    /**
     * get the JonasConnector object for this Rar
     */
    public JonasConnector getJonasConnector() {
        return jonasConn;
    }

    public Object getFactory(String jndiname) {
        ConfigObj co = (ConfigObj) cfgObjs.get(jndiname);
        return (co==null ? null : co.factory );
    }

    public String getInterface(String jndiname) {
        ConfigObj co = (ConfigObj) cfgObjs.get(jndiname);
        return (co==null ? null : co.interfaceStr );
    }

    public Vector getJndinames() {
        return jndinames;
    }

    public ResourceAdapter getResourceAdapter() {
        return resAdp;
    }

    public void configureAS(ActivationSpec as, JLinkedList acp, JLinkedList jacp, String jndiname,
                             String ejbName) throws Exception {
        ConfigProperty [] asCp = buildConfigProperty(null, acp, jacp);
        ((ResourceAdapterAssociation)as).setResourceAdapter(resAdp);
        processSetters(as.getClass(), as, ejbName, asCp);
        validateAS(as, jndiname, acp, jacp);
    }

    //--------------------------------------------------------------
    //           Private Methods
    //--------------------------------------------------------------

    private void validateAS(ActivationSpec as, String jndiname,
                             JLinkedList acp, JLinkedList jacp) throws Exception{
        boolean found = false;
        String reqName = null;
        RequiredConfigProperty rcProp = null;
        ActivationConfigProperty acProp = null;
        // Check that the req properties have been satisfied
        ConfigObj co = (ConfigObj) cfgObjs.get(jndiname);
        for (Iterator r = co.reqConfigProps.iterator(); r.hasNext(); ) {
            rcProp = (RequiredConfigProperty) r.next();
            found = false;
            reqName = rcProp.getConfigPropertyName();
            if (acp != null) {
                for (Iterator i = acp.iterator(); i.hasNext(); ) {
                    acProp = (ActivationConfigProperty) i.next();
                    if (reqName.equals(acProp.getActivationConfigPropertyName())) {
                        found = true;
                        break;
                    }
                }
            }
            if (!found && jacp != null) {
                for (Iterator i = jacp.iterator(); i.hasNext(); ) {
                    acProp = (ActivationConfigProperty) i.next();
                    if (reqName.equals(acProp.getActivationConfigPropertyName())) {
                        found = true;
                        break;
                    }
                }
            }
            if (!found) {
                logger.log(BasicLevel.ERROR, "Required property " + reqName + " not specified ");
                throw new ResourceAllocationException("Required property " + reqName + " not specified ");
            }
        }

        // Call validate
        as.validate();

    }


    // Needs to be implemented
    private void bldSecurityTable(JonasConnector lnkJCon, JonasConnector jCon) {
    }

    public static String buildModuleName(String rarFileName) {
        String sName = null;
        try {
            sName = new File(rarFileName).getName();
            int iPos= sName.lastIndexOf('.');
            if (iPos > -1) {
                sName = sName.substring(0, iPos);
            }
        }
        catch (NullPointerException e) {
            // none action
        }
        return sName;
    }

    private ConfigProperty [] buildConfigProperty(JLinkedList raCfg,
                                                   JLinkedList jRaCfg1,
                                                   JLinkedList jRaCfg2) {
        Vector cfVec = null;
        ConfigProperty[] configs = null;
        if (raCfg != null) {
            for (Iterator i = raCfg.iterator(); i.hasNext(); ) {
                if (cfVec == null) {
                    cfVec = new Vector();
                }
                cfVec.add((ConfigProperty) i.next());
            }
        } else {  //Special case for configuring ActivationSpecs
            ActivationConfigProperty acp = null;
            ConfigProperty cp = null;
            if (jRaCfg1 != null) {
                for (Iterator i = jRaCfg1.iterator(); i.hasNext(); ) {
                    if (cfVec == null) {
                        cfVec = new Vector();
                    }
                    acp = (ActivationConfigProperty) i.next();
                    cp = new ConfigProperty();
                    cp.setConfigPropertyName(acp.getActivationConfigPropertyName());
                    cp.setConfigPropertyValue(acp.getActivationConfigPropertyValue());
                    cfVec.add(cp);
                }
            }

            if (cfVec == null) {
                if (jRaCfg2 != null) {
                    for (Iterator i = jRaCfg2.iterator(); i.hasNext(); ) {
                        if (cfVec == null) {
                            cfVec = new Vector();
                        }
                        acp = (ActivationConfigProperty) i.next();
                        cp = new ConfigProperty();
                        cp.setConfigPropertyName(acp.getActivationConfigPropertyName());
                        cp.setConfigPropertyValue(acp.getActivationConfigPropertyValue());
                        cfVec.add(cp);
                    }
                    configs = new ConfigProperty[cfVec.size()];
                    cfVec.copyInto(configs);
                }
            } else {
                if (jRaCfg2 != null) {
                    boolean found = false;
                    for (Iterator i = jRaCfg2.iterator(); i.hasNext(); ) {
                        found = false;
                        acp = (ActivationConfigProperty) i.next();
                        String name = acp.getActivationConfigPropertyName();
                        String val = acp.getActivationConfigPropertyValue();
                        if (val != null && val.length() > 0) {
                            for (int j=0;j<cfVec.size();j++) {
                                cp = (ConfigProperty) cfVec.get(j);
                                if (name.equalsIgnoreCase(cp.getConfigPropertyName())) {
                                    cp.setConfigPropertyValue(val);
                                    cfVec.set(j, cp);
                                    found = true;
                                    break;
                                }
                            }
                            // This is the case where a new activation-config property is in
                            // the jonas specific dd
                            if (!found) {
                                cp = new ConfigProperty();
                                cp.setConfigPropertyName(name);
                                cp.setConfigPropertyValue(val);
                                cfVec.add(cp);
                            }
                        }
                    }
                }
                configs = new ConfigProperty[cfVec.size()];
                cfVec.copyInto(configs);
            }

            return configs;
        }

        if (cfVec == null) {
            return null;
        }

        configs = new ConfigProperty[cfVec.size()];
        cfVec.copyInto(configs);

        JonasConfigProperty jcpNext = null;
        if (jRaCfg2 != null) {
            for (Iterator i = jRaCfg2.iterator(); i.hasNext(); ){
                jcpNext = (JonasConfigProperty) i.next();
                String name = jcpNext.getJonasConfigPropertyName();
                String val = jcpNext.getJonasConfigPropertyValue();
                if (val != null && val.length() > 0) {
                    for (int j=0;j<configs.length;j++) {
                        if (name.equalsIgnoreCase(configs[j].getConfigPropertyName())) {
                            configs[j].setConfigPropertyValue(val);
                            break;
                        }
                    }
                }
            }
        }

        if (jRaCfg1 != null) {
            for (Iterator i = jRaCfg1.iterator(); i.hasNext(); ){
                jcpNext = (JonasConfigProperty) i.next();
                String name = jcpNext.getJonasConfigPropertyName();
                String val = jcpNext.getJonasConfigPropertyValue();
                if (val != null && val.length() > 0) {
                    for (int j=0;j<configs.length;j++) {
                        if (name.equalsIgnoreCase(configs[j].getConfigPropertyName())) {
                            configs[j].setConfigPropertyValue(val);
                            break;
                        }
                    }
                }
            }
        }

        return configs;
    }

    private Properties buildProp(ConfigProperty [] cfgRaJonas)
    {
      Properties ret = new Properties();
      if (cfgRaJonas != null) {
          for (int i=0;i<cfgRaJonas.length;i++){
              String val = cfgRaJonas[i].getConfigPropertyValue()!=null ? cfgRaJonas[i].getConfigPropertyValue() : "";
              ret.setProperty(cfgRaJonas[i].getConfigPropertyName(), val);
            }
      }
     return ret;
    }

    private boolean checkLogEnabled(String inp) {
        if (inp.equals("1") || inp.equalsIgnoreCase("on") ||
            inp.equalsIgnoreCase("t") || inp.equalsIgnoreCase("true") ||
            inp.equalsIgnoreCase("y") || inp.equalsIgnoreCase("yes"))
            return true;
         return false;
    }

    private ConnectionManager createConnectionManager(String trans) throws Exception {

        ConnectionManagerImpl cm = new ConnectionManagerImpl(trans);

        Context c = new CompNamingContext("");
        c.rebind(ConnectionManagerImpl.TRANSACTION_MANAGER, tm);
        c.rebind(ConnectionManagerImpl.RESOURCE_MANAGER_EVENT_LISTENER,tm);

        // Set the MonologFactory into context of initialisation
        c.rebind(ConnectionManagerImpl.LOGGER, logger);
        c.rebind(ConnectionManagerImpl.POOL_LOGGER, poolLogger);

        // Initialize the ConnectionManager with the configured Context
        cm.init(c);
        return cm;
    }

    /**
     * Process the specified rar, extracting all the jars files to be included
     * in the classloader and all the native files needed by the rar.
     * @param resFileName rar file to process
     * @param jonasConn the jonas-ra.xml properties
     * @throws ResourceServiceException if an error occurs during
     * the extracting.
     */
    private void extractJars(String resFileName, JonasConnector jonasConn)
                 throws ResourceServiceException {

        String jarPath = WORK_RARS_DIR + File.separator + "jonas" + File.separator;
        String binPath = null;
        if (jonasConn.getNativeLib() != null  && jonasConn.getNativeLib().trim().length() > 0) {
            binPath = jonasConn.getNativeLib().trim() + File.separator;
            if (binPath.startsWith("/") || binPath.startsWith("\\") ||
                binPath.charAt(1) == ':') {
                ;
            } else {
                binPath = JProp.getJonasBase() + File.separator + binPath;
            }

        }

        //Get RAR name
        String rarName = resFileName;
        if (rarName.endsWith(".rar")) {
            rarName = resFileName.substring(0, resFileName.lastIndexOf('.'));
        }

        int off = rarName.lastIndexOf(File.separator);
        rarName = rarName.substring(++off);
        if (File.separatorChar == '\\') {
            off = rarName.lastIndexOf("/");
            rarName = rarName.substring(++off);
        }

        JJarFile jjar = null;

        try {
            jjar = new JJarFile(resFileName);
            for (Enumeration ent = jjar.entries(); ent.hasMoreElements(); ) {
                JarEntry je = (JarEntry) ent.nextElement();
                if (!je.isDirectory() && je.getName().endsWith(".jar")) {
                    File newd = new File(jarPath);
                    if (!newd.exists()) {
                        newd.mkdirs();
                    }

                    String fileName = jarPath+"_"+rarName+"_"+je.getName();
                    jjar.extract(je, fileName);
                    curLoader.addURL((new File(fileName)).toURL());
                } else if (!je.isDirectory() && !je.getName().startsWith("META-INF") &&
                            binPath != null){
                    File newf = new File(binPath);
                    if (!newf.exists()) {
                        newf.mkdirs();
                    }
                    String fileName = binPath+je.getName();
                    jjar.extract(je, fileName);
                }
            }
        } catch (Exception ex) {
            String err = "Error while extracting the files from "+resFileName;
            logger.log(BasicLevel.ERROR, err + ex.getMessage());
            throw new ResourceServiceException(err, ex);
        }

    }

    /**
     * get the ResourceAdapter Connector object matching the jndiName
     */
    private Connector getConnector(String jndiName) {
        return ResourceServiceImpl.getConnector(jndiName);
    }

    /**
     * get the JonasConnector object matching the jndiName
     */
    private JonasConnector getJonasConnector(String jndiName) {
        return ResourceServiceImpl.getJonasConnector(jndiName);
    }

    /**
     * get the ResourceAdapter Connector object matching the jndiName
     */
    private String getXmlContent(String jndiName) {
        return ResourceServiceImpl.getXmlContent(jndiName);
    }

    private Object getJonasXML(JonasConnector jonasConn, String id, int idOffset, String oType) {
        Object obj = null;
        if (oType == JCD) {
            obj = jonasConn.getJonasConnectionDefinitionList().get(idOffset);
        } else if (oType == JAS) {
            obj = jonasConn.getJonasActivationspecList().get(idOffset);
        } else {
            obj = jonasConn.getJonasAdminobjectList().get(idOffset);
        }
        return obj;
    }

    private void processJAO(Adminobject aObj, JonasAdminobject jao, ConfigProperty [] cfgRaJonas,
                             int idOff) throws Exception{

        ConfigObj cObj = null;
        String jndiName = jao.getJndiName();
        if (jndiName == null || jndiName.length() == 0){
            logger.log(BasicLevel.ERROR,"Rar: jndi-name not set in jonas-ra.xml for Adminobject: " + idOff);
            throw new Exception("configuration file incorrect");
        }

        // Instantiate into the currect loader
        String admObj = aObj.getAdminobjectClass();
        Class aoClass = curLoader.loadClass(admObj);
        Object ao = aoClass.newInstance();

        processSetters(aoClass, ao, rarFileName, cfgRaJonas);

        jndinames.add(jndiName);
        logger.log(BasicLevel.DEBUG, "jndiName="+ jndiName);

        // Add entry to list of configured objects
        cObj = new ConfigObj(JAO, idOff, jndiName, aObj.getAdminobjectInterface(), admObj, ao);

        cfgObjs.put(jndiName, cObj);
        //
        // Register into Jndi
        try {

            Reference ref =
                new Reference(
                              ao.getClass().getName(),
                              ResourceObjectJNDIHandler.class.getName(),
                              null);
            ref.add( new StringRefAddr(JNDI_NAME, jndiName) );
            ref.add( new StringRefAddr(OBJ_TYPE, JAO) );
            ictx.rebind(jndiName, ao);

        } catch (Exception e) {
            logger.log(BasicLevel.ERROR,"Rar: Cannot register ResourceAdapter with the name " + jndiName);
            logger.log(BasicLevel.ERROR,"Rar: Exception caught : "+e);
            return;
        }

        // --------------------------
        // Register MBeans cf. JSR 77
        // --------------------------
        if (mbeanServer != null) {

            // Available Adminobjects
            // ------------------------------------------------------------
            // Adminobject MBean
            // -----------------------
            String jcaAdminobjectName = jndiName;
            ObjectName onJCAAdminobject =
                J2eeObjectName.getJCAAdminObject(jDomain,
                                                 JCAResourceName,
                                                 jServer,
                                                 jcaAdminobjectName);
            JCAAdminObject jcaAdminobjectMBean = new JCAAdminObject(onJCAAdminobject.toString());
            ManagedBean oManaged = oRegistry.findManagedBean("JCAAdminObject");
            ModelMBean oMBean = oManaged.createMBean(jcaAdminobjectMBean);
            if (manageLogger.isLoggable(BasicLevel.DEBUG)) {
                manageLogger.log(BasicLevel.DEBUG, "JCAAdminObject created");
            }
            mbeanServer.registerMBean(oMBean, onJCAAdminobject);

            // Update the list of connection factories in the JCAResource MBean with the JCAConnectionFactory
            // MBean's OBJECT_NAME
            jcaResourceMBean.setAdminObjects(onJCAAdminobject.toString());

        } // end JMX registration

    }

    private void processJAS(Messagelistener mlst, JonasActivationspec jas,
                             int idOff) throws Exception{

        ConfigObj cObj = null;
        String jndiName = jas.getJndiName();
        if (jndiName == null || jndiName.length() == 0){
            logger.log(BasicLevel.ERROR,"Rar: jndi-name not set in jonas-ra.xml for Activationspec: " + idOff);
            throw new Exception("configuration file incorrect");
        }

        // Instantiate into the currect loader
        String mlIntf = mlst.getMessagelistenerType();
        String actSpec = mlst.getActivationspec().getActivationspecClass();
        Class asClass = curLoader.loadClass(actSpec);
        ActivationSpec as =
            (ActivationSpec) asClass.newInstance();

        jndinames.add(jndiName);
        logger.log(BasicLevel.DEBUG, "jndiName=" + jndiName);

        // Add entry to list of configured objects
        cObj = new ConfigObj(JAS, idOff, jndiName, mlIntf, actSpec, as);
        cObj.reqConfigProps = mlst.getActivationspec().getRequiredConfigPropertyList();

        cfgObjs.put(jndiName, cObj);
        //
        // Register into Jndi
        try {

            Reference ref =
                new Reference(
                              as.getClass().getName(),
                              ResourceObjectJNDIHandler.class.getName(),
                              null);
            ref.add( new StringRefAddr(JNDI_NAME, jndiName) );
            ref.add( new StringRefAddr(OBJ_TYPE, JAS) );
            ictx.rebind(jndiName, as);

        } catch (Exception e) {
            logger.log(BasicLevel.ERROR,"Rar.processJAS: Cannot register ResourceAdapter with the name " + jndiName);
            logger.log(BasicLevel.ERROR,"Rar.processJAS: Exception caught : "+e);
            return;
        }

        // --------------------------
        // Register MBeans cf. JSR 77
        // --------------------------
        if (mbeanServer != null) {

            // Available ActivationSpecs
            // ------------------------------------------------------------
            // ActivationSpec MBean
            // -----------------------
            String jcaActivationSpecName = jndiName;
            ObjectName onJCAActivationSpec =
                J2eeObjectName.getJCAActivationSpec(jDomain,
                                                       JCAResourceName,
                                                       jServer,
                                                       jcaActivationSpecName);
            JCAActivationSpec jcaActivationSpecMBean = new JCAActivationSpec(onJCAActivationSpec.toString());
            ManagedBean oManaged = oRegistry.findManagedBean("JCAActivationSpec");
            ModelMBean oMBean = oManaged.createMBean(jcaActivationSpecMBean);
            if (manageLogger.isLoggable(BasicLevel.DEBUG)) {
                manageLogger.log(BasicLevel.DEBUG, "JCAActivationSpec created");
            }
            mbeanServer.registerMBean(oMBean, onJCAActivationSpec);

            // Update the list of connection factories in the JCAResource MBean with the JCAConnectionFactory
            // MBean's OBJECT_NAME
            jcaResourceMBean.setActivationSpecs(onJCAActivationSpec.toString());

        } // end JMX registration

    }

    private void processMCF(String mcfc, String transSupport,
                             String jndiName, String logEnabled, String logTopic,
                             ConfigProperty [] cfgRaJonas, intPoolParams pParams,
                             boolean basicPass, int idOff) throws Exception{

        ConfigObj cObj = null;
        ConnectionManager cm = null;
        if (mcfc == null){
            logger.log(BasicLevel.ERROR,"ResourceService.createRA:"+MANAGED_CONNECTION_FACTORY_CLASS+" property not found");
            throw new Exception("configuration file incorrect");
        }

        if (jndiName == null || jndiName.length() == 0){
            logger.log(BasicLevel.ERROR,"ResourceService.createRA: jndi-name not set in jonas-ra.xml");
            throw new Exception("configuration file incorrect");
        }

        // Instantiate into the currect loader
        Class mcfClass = curLoader.loadClass(mcfc);
        ManagedConnectionFactory mcf =
            (ManagedConnectionFactory) mcfClass.newInstance();

        if (resAdp != null) {
            try {
                ((ResourceAdapterAssociation) mcf).setResourceAdapter(resAdp);
            } catch (ClassCastException ce) {
                // Not able to associate ResourceAdapter to MCF
            } catch (Exception ex) {
                logger.log(BasicLevel.ERROR,"ResourceService: Error setting ResourceAdapter class to ManagedConnectionFactory ("+mcfc+") for " + jndiName);
                throw ex;
            }
        }

        cm = createConnectionManager(transSupport);

        jndinames.add(jndiName);
        logger.log(BasicLevel.DEBUG, "jndiName="+ jndiName);

        // Check if logging desired and the value of the desired topic
        if (logEnabled != null) {
            logger.log(BasicLevel.DEBUG, "log-enabled="+ logEnabled);
            if (checkLogEnabled(logEnabled)) {
                if (logTopic != null && logTopic.length() > 0) {
                    logger.log(BasicLevel.DEBUG, "log-topic="+ logTopic);
                    mcf.setLogWriter(Log.getLogWriter(logTopic));
                } else {
                    logger.log(BasicLevel.DEBUG, "default log-topic="+ Log.JONAS_JCA_PREFIX);
                    mcf.setLogWriter(Log.getLogWriter(Log.JONAS_JCA_PREFIX));   // "org.objectweb.jonas.jca"
                }
            }
        }

        processSetters(mcfClass, mcf, rarFileName, cfgRaJonas);
        
        PrintWriter pw = mcf.getLogWriter();
        if (pw != null) {
            //Create a date format
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy.MM.dd HH:mm:ss z");
            String date = sdf.format(new Date());
            pw.println("MCF: output starting at "+date);
            pw.flush();
        }

        // Set hashCode for ManagedConnectionFactory
        mcf.hashCode();

        // Obtain the CF and connect the RA to the ConnectionManager
        // A connection factory must implements javax.resource.Referenceable
        Referenceable cf = (Referenceable)
            mcf.createConnectionFactory(cm);

        logger.log(BasicLevel.DEBUG, "cf="+ cf);

        ((ConnectionManagerImpl) cm).setResourceAdapter(mcf, pParams.poolInit, pParams.poolMin, pParams.poolMax,
                                                        pParams.pstmtMax, pParams.maxage, pParams.jdbcConnLevel,
                                                        pParams.jdbcConnTestStmt);
        // Add entry to list of configured objects
        cObj = new ConfigObj(JCD, idOff, jndiName, null, mcfc, cf);
        cObj.cm = cm;
        cObj.basicPass = basicPass;

        cfgObjs.put(jndiName, cObj);
        //
        // Register into Jndi
        try {

            Reference ref =
                new Reference(
                              cf.getClass().getName(),
                              ResourceObjectJNDIHandler.class.getName(),
                              null);
            ref.add( new StringRefAddr(JNDI_NAME, jndiName) );
            cf.setReference(ref);
            ictx.rebind(jndiName, cf);

        } catch (Exception e) {
            logger.log(BasicLevel.ERROR,"ResourceService: Cannot register ResourceAdapter in naming with the name " + jndiName);
            logger.log(BasicLevel.ERROR,"ResourceService: Exception caught : "+e);
            return;
        }

        // --------------------------
        // Register MBeans cf. JSR 77
        // --------------------------
        if (mbeanServer != null) {

            // Available ConnectionFactories and ManagedConnectionFactories
            // ------------------------------------------------------------
            // ConnectionFactory MBean
            // -----------------------
            String jcaConnectionFactoryName = jndiName;
            ObjectName onJCAConnectionFactory =
                J2eeObjectName.getJCAConnectionFactory(jDomain,
                                                       JCAResourceName,
                                                       jServer,
                                                       jcaConnectionFactoryName);
            JCAConnectionFactory jcaConnectionFactoryMBean = new JCAConnectionFactory(onJCAConnectionFactory.toString());
            ManagedBean oManaged = oRegistry.findManagedBean("JCAConnectionFactory");
            ModelMBean oMBean = oManaged.createMBean(jcaConnectionFactoryMBean);
            if (manageLogger.isLoggable(BasicLevel.DEBUG)) {
                manageLogger.log(BasicLevel.DEBUG, "JCAConnectionFactory created");
            }
            mbeanServer.registerMBean(oMBean, onJCAConnectionFactory);

            // Update the list of connection factories in the JCAResource MBean with the JCAConnectionFactory
            // MBean's OBJECT_NAME
            jcaResourceMBean.setConnectionFactory(onJCAConnectionFactory.toString());

            // ManagedConnectionFactory MBean
            // ------------------------------
            String jcaManagedConnectionFactoryName = "aManagedConnectionFactory-" + jcaConnectionFactoryName; // TO BE IMPLEMENTED
            ObjectName onJCAManagedConnectionFactory =
                J2eeObjectName.getJCAManagedConnectionFactory(jDomain,
                                                       jServer,
                                                       jcaManagedConnectionFactoryName);
            JCAManagedConnectionFactory jcaManagedConnectionFactoryMBean =
                new JCAManagedConnectionFactory(onJCAManagedConnectionFactory.toString());
            oManaged = oRegistry.findManagedBean("JCAManagedConnectionFactory");
            oMBean = oManaged.createMBean(jcaManagedConnectionFactoryMBean);
            if (manageLogger.isLoggable(BasicLevel.DEBUG)) {
                manageLogger.log(BasicLevel.DEBUG, "JCAManagedConnectionFactory created");
            }
            mbeanServer.registerMBean(oMBean, onJCAManagedConnectionFactory);

            // Update the JCA ConnectionFactory with the JCA ManagedConnectionFactory
            jcaConnectionFactoryMBean.setManagedConnectionFactory(onJCAManagedConnectionFactory.toString());
            if (manageLogger.isLoggable(BasicLevel.DEBUG)) {
                manageLogger.log(BasicLevel.DEBUG, "JCAConnectionFactory updated");
            }

        } // end JMX registration

    }

    /**
     *
     */
    private void processSetters(Class clsClass, Object clsObj, String rarFileName,
                                 ConfigProperty [] cProp) throws Exception{

        int curParam = 0;
        while ( cProp != null && curParam < cProp.length ){
            String fieldName = cProp[curParam].getConfigPropertyName();
            String methodName = "set"+fieldName.substring(0,1).toUpperCase() +
                                fieldName.substring(1);
            String fieldType = cProp[curParam].getConfigPropertyType();
            Method[] m = clsClass.getMethods();

            int i=0;
            while (i<m.length
                   && (! m[i].getName().equals(methodName)
                       || m[i].getParameterTypes().length!=1
                       || (fieldType != null &&
                           ! (m[i].getParameterTypes())[0].getName().equals(fieldType)))
                   ) {
                i++;
            }
            if (i<m.length) {
                Class [] paramtype = m[i].getParameterTypes();
                Object[] param = new Object[1];

                String curValue = cProp[curParam].getConfigPropertyValue();

                setterLogger.log(BasicLevel.DEBUG, "Processing Field Name: "+fieldName+" Type: "+fieldType+" Value: "+curValue);

                if (paramtype[0].equals(java.lang.Integer.TYPE) ||
                    paramtype[0].equals(java.lang.Integer.class)){
                    param[0]  = new Integer(curValue);

                } else if (paramtype[0].equals(java.lang.Boolean.TYPE) ||
                           paramtype[0].equals(java.lang.Boolean.class) ){
                    param[0]  = new Boolean(curValue);

                } else if (paramtype[0].equals(java.lang.Double.TYPE) ||
                           paramtype[0].equals(java.lang.Double.class) ){
                    param[0]  = new Double(curValue);

                } else if (paramtype[0].equals(java.lang.Byte.TYPE) ||
                           paramtype[0].equals(java.lang.Byte.class) ){
                    param[0]  = new Byte(curValue);

                } else if (paramtype[0].equals(java.lang.Short.TYPE) ||
                           paramtype[0].equals(java.lang.Short.class)){
                    param[0]  = new Short(curValue);

                } else if (paramtype[0].equals(java.lang.Long.TYPE) ||
                           paramtype[0].equals(java.lang.Long.class) ){
                    param[0]  = new Long(curValue);

                } else if (paramtype[0].equals(java.lang.Float.TYPE) ||
                           paramtype[0].equals(java.lang.Float.class) ){
                    param[0]  = new Float(curValue);

                } else if (paramtype[0].equals(java.lang.Character.TYPE) ||
                            paramtype[0].equals(java.lang.Character.class) ){
                    param[0]  = new Character(curValue.charAt(0));

                } else if (paramtype[0].equals(java.lang.String.class)){
                    param[0]  = curValue;
                    setterLogger.log(BasicLevel.DEBUG, "Calling String method with "+curValue);

                } else {
                    logger.log(BasicLevel.ERROR,"Type unsupported for setter method:"
                                +methodName);
                    throw new Exception("incorrect type for setter method ");
                }
                // invocation of the corresponding setter method on
                // class object instance
                try {
                    m[i].invoke(clsObj, param);
                } catch (Exception e) {
                    logger.log(BasicLevel.DEBUG, "method "
                                +methodName+
                                " not found for " + rarFileName + "check its configuration ");
                }
                curParam++;
            } else {
                logger.log(BasicLevel.ERROR,"Method not found: "+methodName);
                throw new Exception("incorrect method name: "+methodName);
            }
        }
    }

    /**
     * set the PoolParams from the JonasConnector specified
     */
    private void setPoolParams (PoolParams pParams, JdbcConnParams jConnParams,
                                intPoolParams pool) throws Exception{

        String tmpPool = null;
        try {
            if (pParams != null) {
                if (pParams.getPoolInit() != null) {
                    tmpPool = pParams.getPoolInit();
                    if (tmpPool != null && tmpPool.length() > 0) pool.poolInit = Integer.parseInt(tmpPool);
                }
                if (pParams.getPoolMin() != null) {
                    tmpPool = pParams.getPoolMin();
                    if (tmpPool != null && tmpPool.length() > 0) pool.poolMin = Integer.parseInt(tmpPool);
                }
                if (pParams.getPoolMax() != null) {
                    tmpPool = pParams.getPoolMax();
                    if (tmpPool != null && tmpPool.length() > 0) pool.poolMax = Integer.parseInt(tmpPool);
                }
                if (pParams.getPstmtMax() != null) {
                    tmpPool = pParams.getPstmtMax();
                    if (tmpPool != null && tmpPool.length() > 0) pool.pstmtMax = Integer.parseInt(tmpPool);
                }
                if (pParams.getPoolMaxAge() != null) {
                    tmpPool = pParams.getPoolMaxAge();
                    if (tmpPool != null && tmpPool.length() > 0) pool.maxage = Long.parseLong(tmpPool);
                }
            }
            if (jConnParams != null) {
                if (jConnParams.getJdbcCheckLevel() != null) {
                    tmpPool = jConnParams.getJdbcCheckLevel();
                    if (tmpPool != null && tmpPool.length() > 0) pool.jdbcConnLevel = Integer.parseInt(tmpPool);
                }
                if (jConnParams.getJdbcTestStatement() != null) {
                    pool.jdbcConnTestStmt = jConnParams.getJdbcTestStatement();
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            logger.log(BasicLevel.ERROR,"Invalid Pool parameter from jonas-ra.xml:  Ensure that numeric values are used. "+tmpPool);
            throw new Exception("incorrect pool parameter ");
        }
    }


}


