/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * --------------------------------------------------------------------------
 * $Id: J2EEServerMBean.java,v 1.13 2004/06/09 16:41:13 camillej Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.server;

// Java imports
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.rmi.RemoteException;
import java.util.ArrayList;
	
import javax.management.MBeanException;
import javax.management.Notification;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;

import org.apache.commons.modeler.BaseModelMBean;
import org.objectweb.common.Cmd;
import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.container.EJBServiceImpl;
import org.objectweb.jonas.ear.EarServiceException;
import org.objectweb.jonas.ear.EarServiceImpl;
import org.objectweb.jonas.resource.ResourceServiceException;
import org.objectweb.jonas.resource.ResourceServiceImpl;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.service.ServiceManager;
import org.objectweb.jonas.web.AbsJWebContainerServiceImpl;
import org.objectweb.jonas.web.JWebContainerServiceException;
import org.objectweb.jonas_ear.deployment.api.EarDeploymentDesc;
import org.objectweb.jonas_ear.deployment.api.EarDeploymentDescException;
import org.objectweb.jonas_ear.deployment.lib.wrapper.EarManagerWrapper;
import org.objectweb.jonas_ejb.genic.wrapper.GenicServiceWrapper;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
/**
 * MBean class for Server management.
 * Two classes are used to provide all management services : J2EEServer and J2EEServerMBean.
 * This class provide the notification's process when add or remove resources.
 * @author Adriana Danes
 * @author Michel-Ange Anton
 */

public class J2EEServerMBean extends BaseModelMBean {

// ------------------------------------------------------------- Private variables

    private static Logger mgtLogger = Log.getLogger(Log.JONAS_MANAGEMENT_PREFIX);

// ------------------------------------------------------------- Contructors

    public J2EEServerMBean()
        throws MBeanException {
        super();
    }

// ------------------------------------------------------------- Public methods

    /**
     * All The MBean names corresponding to the deployed J2EEModules (ear, jar, war, rar).
     * @return The String's array of deployed modules objects names
     */
    public String[] getDeployedObjects() {
        ArrayList al = (ArrayList) ((J2EEServer) (this.resource)).getDeployedObjects();
        return (String[]) al.toArray(new String[al.size()]);
    }

    /**
     * All The MBean names corresponding to the deployed J2EEResources.
     * @return The String's array of deployed resources objects names
     */
    public String[] getResources() {
        ArrayList al = (ArrayList) ((J2EEServer) (this.resource)).getResources();
        return (String[]) al.toArray(new String[al.size()]);
    }

    /**
     * All The MBean names corresponding to the used JVMs.
     * @return The String's array of used JVMs objects names
     */
    public String[] getJavaVMs() {
        ArrayList al = (ArrayList) ((J2EEServer) (this.resource)).getJavaVMs();
        return (String[]) al.toArray(new String[al.size()]);
    }

    /**
     * Send a notification to the listener.
     * @param pNotification The notification to send
     */
    public void sendNotification(Notification pNotification) {
        ((J2EEServer) (this.resource)).sendNotification(pNotification);
    }

    /**
     * Add a new listener.
     * @param pListner Listener to notify
     * @param pFilter Notification filter
     * @param pHandback ??
     * @throws java.lang.IllegalArgumentException
     */
    public void addNotificationListener(NotificationListener pListner, NotificationFilter pFilter
        , java.lang.Object pHandback)
        throws java.lang.IllegalArgumentException {
        ((J2EEServer) (this.resource)).addNotificationListener(pListner, pFilter, pHandback);
    }

    /**
     * Deploy a stand-alone J2EE module packaged in a JAR file
     * @param fileName file name
     * @return The ObjectName of the MBean associated to the deployed module
     * @throws Exception Management operation failed
     */
    public String deployJar(String fileName) throws Exception {
        return ((J2EEServer) (this.resource)).deployJar(fileName);
    }

    /**
     * Deploy a J2EE application packaged in a EAR file
     * @param fileName file name
     * @return The ObjectName of the MBean associated to the deployed J2EE Application
     * @throws Exception Management operation failed
     */
    public String deployEar(String fileName)  throws Exception {
        return ((J2EEServer) (this.resource)).deployEar(fileName);
    }

    /**
     * Deploy a J2EE application packaged in a RAR file
     * @param fileName file name
     * @return The ObjectName of the MBean associated to the deployed J2EE Application
     * @throws Exception Management operation failed
     */
    public String deployRar(String fileName)  throws Exception {
        return ((J2EEServer) (this.resource)).deployRar(fileName);
    }

    /**
     * Test if the specified filename is already deployed or not.
     * @param fileName the name of the ear file.
     * @return true if the ear is deployed, otherwise false.
     * @throws Exception Management operation failed
     */
    public Boolean isEarDeployed(String fileName) throws Exception {
        return ((J2EEServer) (this.resource)).isEarDeployed(fileName);
    }

    /**
     * Test if the specified filename is already deployed or not.
     * @param fileName the name of the rar file.
     * @return true if the rar is deployed, otherwise false.
     * @throws Exception Management operation failed
     */
    public Boolean isRarDeployed(String fileName) throws Exception {
        return ((J2EEServer) (this.resource)).isRarDeployed(fileName);
    }

    /**
     * Test if the specified filename is already deployed or not.
     * @param fileName the name of the jar file.
     * @return true if the jar is deployed, otherwise false.
     * @throws Exception Management operation failed
     */
    public Boolean isJarDeployed(String fileName) throws Exception {
        return ((J2EEServer) (this.resource)).isJarDeployed(fileName);
    }
    /**
     * Deploy file (GenIC), needed for Ishmael to work. The file is in the directory
     * specified by the property "jonas.service.deployment.directory"
     * @param type type of the file (EJB, WAR, EAR, RAR, CAR)
     * @param bfile
     * @param filename basename of the file to be deployed
     * @throws RemoteException
     * @throws EarServiceException
     * @throws ResourceServiceException
     * @throws JWebContainerServiceException
     * @author Dean Jennings. <da_jennings@junta.com.au>
     */
    public String deployFile(Integer typeparam, java.lang.Byte[] bfile, String filename) {
        String directory = "";
        // Patch needed to compile
        int TYPE_EJB = 1;
        int TYPE_WAR = 2;
        int TYPE_EAR = 3;
        int TYPE_RAR = 4;
        int TYPE_CAR = 5;
        
        int type = typeparam.intValue();
       
        ServiceManager serviceManager = null;
        try {
            serviceManager = ServiceManager.getInstance();
        } catch (Exception e) {
            // TODO 
        }
        if (type == TYPE_EJB) {
            directory = ((EJBServiceImpl) serviceManager.getEjbService()).getEjbjarsDirectory();
        } else if (type == TYPE_EAR) {
            directory = ((EarServiceImpl) serviceManager.getEarService()).getAppsDirectory();
        } else if (type == TYPE_WAR) {
            directory = ((AbsJWebContainerServiceImpl) serviceManager.getWebContainerService()).getWebappsDirectory();
        } else if (type == TYPE_RAR) {
            directory = ((ResourceServiceImpl) serviceManager.getResourceService()).getRarsDirectory();
        } else if (type == TYPE_CAR) {
            // TO DO
            //throw new UnsupportedOperationException("Not Supported yet");
        }
        
        File file = new File(directory + filename);
        try {
            FileOutputStream out = new FileOutputStream(file);
            byte[] bfileused = new byte[bfile.length];
            for (int i = 0; i < bfile.length; i++) {
              bfileused[i] = bfile[i].byteValue();
            }
            out.write(bfileused);
            out.close();
            if (type == TYPE_EJB) {
                String args[] = new String[1];
                args[0] = directory + filename;
                try {
                   GenicServiceWrapper.callGenic(args);
                } catch (ServiceException e) {
                    mgtLogger.log(BasicLevel.WARN
                            , "deploy File error " + filename
                            + e.getClass().getName() + " " + e.getMessage());
                }
            } else if (type == TYPE_EAR) {
                unpackAndCompileEar(file);
            } else {
                // TO DO ??
            }
        } catch (IOException ioe) {
            // TO DO
        }
        
        return file.getAbsolutePath();
    }
    
    private void unpackAndCompileEar(File file) throws ServiceException {
        URL earUrl[] = new URL[1];

        try {
            earUrl[0] = file.toURL();
        } catch (MalformedURLException e) {
            // TO DO
            //String err = "Invalid ear file name '" + file;
            //throw new EarServiceException(err, e);
        }

        //Create classLoader
        //parent classloader is the current classloader
        ClassLoader currentLoader = Thread.currentThread().getContextClassLoader();
        URLClassLoader loaderCls = new URLClassLoader(earUrl, currentLoader);

        EarDeploymentDesc desc = null;
        try {
            desc = EarManagerWrapper.getDeploymentDesc(earUrl[0].getFile(), loaderCls);
        } catch (EarDeploymentDescException e) {
            // TO DO
        }

        String[] ejbTags = desc.getEjbTags();
        /*
         * Example of a jar command:
         *   jar -uf ../output/ejbjars/sb.jar -C /tmp/genic1547.tmp a/b/C1.class
         *                                    -C /tmp/genic1547.tmp a/b/C2.class
         *                                    .....
         */
        String javaHomeBin = System.getProperty("java.home", "");
        if (!("".equals(javaHomeBin))) {
            javaHomeBin = javaHomeBin + File.separator + ".."
                + File.separator + "bin" + File.separator;
        }

        Cmd cmd = null;
        cmd = new Cmd(javaHomeBin + "jar");
        cmd.addArgument("-xf");
        cmd.addArgument(file.getAbsolutePath());

        for (int i = 0; i < ejbTags.length; i++) {
            cmd.addArgument(ejbTags[i]);
        }

        boolean exitCmd = cmd.run();
        // Analyse the result of the jar command
        if (!exitCmd) {
            throw new EarServiceException("Failed when extracting the the ejb jar "
                                          + "in the given jar file '" + file + "'.");
        }

        for (int i = 0; i < ejbTags.length; i++) {
            String args[] = new String[1];
            args[0] = ejbTags[i];
            try {
                GenicServiceWrapper.callGenic(args);
             } catch (ServiceException e) {
             	throw(e);
             }        }

        cmd = new Cmd(javaHomeBin + "jar");
        cmd.addArgument("-uf");
        cmd.addArgument(file.getAbsolutePath());

        for (int i = 0; i < ejbTags.length; i++) {
            cmd.addArgument(ejbTags[i]);
        }
        exitCmd = cmd.run();
        // Analyse the result of the jar command
        if (!exitCmd) {
            throw new EarServiceException("Failed when extracting the the ejb jar "
                                          + "in the given jar file '" + file + "'.");
        }

        for (int i = 0; i < ejbTags.length; i++) {
            File del = new File(ejbTags[i]);
            del.delete();
        }

    } 
    
    /**
     * Add a deployed object in the list of the J2EEServer.
     * @param pDeployedObject The object name of deployed object
     */
    public static void addDeployedObject(String pDeployedObject) {
        try {
            Server oServer = Server.getInstance();
            J2EEServer oJ2EEServer = oServer.getJ2EEServer();
            oJ2EEServer.addDeployedObject(pDeployedObject);
        } catch (Exception e) {
            mgtLogger.log(BasicLevel.WARN
                , "Add deployed object error (object=" + pDeployedObject + ") : "
                + e.getClass().getName() + " " + e.getMessage());
        }
    }

    /**
     * Remove a deployed object from the list of the J2EEServer.
     * @param pDeployedObject The object name of deployed object
     */
    public static void removeDeployedObject(String pDeployedObject) {
        try {
            Server oServer = Server.getInstance();
            J2EEServer oJ2EEServer = oServer.getJ2EEServer();
            oJ2EEServer.removeDeployedObject(pDeployedObject);
        } catch (Exception e) {
            mgtLogger.log(BasicLevel.WARN
                , "Remove deployed object error (object=" + pDeployedObject + ") : "
                + e.getClass().getName() + " " + e.getMessage());
        }
    }

    /**
     * Add an object name to the <code>resources</code> list.
     * @param pObjectName Object name correspondig to a J2EEResource MBean
     */
    /*
    public static void addResource(String pObjectName) {
        try {
            Server oServer = Server.getInstance();
            J2EEServer oJ2EEServer = oServer.getJ2EEServer();
            oJ2EEServer.addResource(pObjectName);
        } catch (Exception e) {
            mgtLogger.log(BasicLevel.WARN
                , "Add resource error (object=" + pObjectName + ") : "
                + e.getClass().getName() + " " + e.getMessage());
        }
    }
*/
    /**
     * Remove an object name from the <code>resources</code> list.
     * @param pObjectName Object name correspondig to a J2EEResource MBean
     * @return Object name to the removed J2EEResource MBean
     */
    /*
    public static String removeResource(String pObjectName) {
        String sRet = null;
        try {
            Server oServer = Server.getInstance();
            J2EEServer oJ2EEServer = oServer.getJ2EEServer();
            sRet = oJ2EEServer.removeResource(pObjectName);
        } catch (Exception e) {
            mgtLogger.log(BasicLevel.WARN
                , "Remove resource object error (object=" + pObjectName + ") : "
                + e.getClass().getName() + " " + e.getMessage());
        }
        return sRet;
    }
*/
}
