/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * --------------------------------------------------------------------------
 * $Id: JavaVm.java,v 1.6 2004/06/07 07:24:22 danesa Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.server;

// Java imports
import org.objectweb.jonas.common.JProp;
import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.management.j2eemanagement.J2EEManagedObject;
import org.objectweb.util.monolog.api.Logger;

/**
 * The Java Virtual Machine used by the JOnAS server.
 * @author Michel-Ange Anton
 */
public class JavaVm extends J2EEManagedObject {

// ------------------------------------------------------------- Private Constants
    /** JRE version property name*/
    private static final String JAVA_VERSION = "java.version";
    /** JRE vendor property name*/
    private static final String JAVA_VENDOR = "java.vendor";
    /** default host name */
    private static final String NODE_LOCAL = "localhost";

// ------------------------------------------------------------- Privates Variables
    /** private logger */
    private static Logger sLogger = null;

// ------------------------------------------------------------- Properties

    /**
     * JVM version.
     */
    private String javaVersion = null;

    /**
     * JVM vendor.
     */
    private String javaVendor = null;

    /**
     * Node (machine) the JVM is running on.
     */
    private String node = null;

// ------------------------------------------------------------- Contructors
    /**
     * Construct JavaVM MBean
     * @param pObjectName The MBean's OBJECT_NAME
     * @param pProps JProp object containing current JVM's properties
     */
    public JavaVm(String pObjectName, JProp pProps) {
        super(pObjectName);
        javaVersion = pProps.getValue(JAVA_VERSION);
        javaVendor = pProps.getValue(JAVA_VENDOR);
        try {
            java.net.InetAddress oInet = java.net.InetAddress.getLocalHost();
            node = oInet.getCanonicalHostName();
        } catch (java.net.UnknownHostException e) {
            node = NODE_LOCAL;
        }
        // get a logger for server traces
        sLogger = Log.getLogger(Log.JONAS_SERVER_PREFIX);
    }

// ------------------------------------------------------------- Properties Methods
    /**
     * @return The JVM version.
     */
    public String getJavaVersion() {
        return javaVersion;
    }
    /**
     * @return The JVM vendor.
     */
    public String getJavaVendor() {
        return javaVendor;
    }

    /**
     * @return The node (machine) the JVM is running on.
     */
    public String getNode() {
        return node;
    }

}
