/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ServletEndpoints.java,v 1.1 2004/07/01 14:57:25 sauthieg Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.ws;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

/**
 * The <code>ServletEndpoints</code> objects are used to configure the JAX-RPC
 * Runtime for endpoints URL linked to ports.
 * @author Guillaume Sauthier
 */
public class ServletEndpoints {

    /**
     * map port names to runtime endpoint URL
     */
    private Map endpoints = null;

    /**
     * Default constructor
     */
    public ServletEndpoints() {
        endpoints = new Hashtable();
    }

    /**
     * @return Returns an Iterator over the port names
     */
    public Iterator getPortNames() {
        return endpoints.keySet().iterator();
    }

    /**
     * @param port the port name we want the endpoint URL
     * @return Returns the enpoint URL associated with the given port name
     */
    public String getEndpointURL(String port) {
        return (String) endpoints.get(port);
    }

    /**
     * Add a map entry associating the given port name to the given URL endpoint
     * @param port the port name used as a key
     * @param url the endpoint URL of the given port
     */
    public void addEndpoint(String port, String url) {
        endpoints.put(port, url);
    }
}