/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): Lutris Tecnologies Inc.
 *
 * --------------------------------------------------------------------------
 * $Id: MessageDrivenJMS2_0Desc.java,v 1.3 2004/05/24 04:40:40 ehardesty Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.deployment.api;

import java.util.Iterator;

import org.objectweb.jonas_ejb.deployment.xml.AssemblyDescriptor;
import org.objectweb.jonas_ejb.deployment.xml.JonasMessageDriven;
import org.objectweb.jonas_ejb.deployment.xml.MessageDriven;
import org.objectweb.jonas_ejb.deployment.xml.MessageDrivenDestination;
import org.objectweb.jonas_lib.deployment.api.DeploymentDescException;
import org.objectweb.jonas_lib.deployment.xml.JLinkedList;

/**
 * Class to hold meta-information related to a message driven bean
 * @author Philippe Coq
 * @author Helene Joanin
 * @author Christophe Ney [cney@batisseurs.com]
 */
public class MessageDrivenJMS2_0Desc extends MessageDrivenDesc {



    /**
     * package protected constructor used by API
     */
    MessageDrivenJMS2_0Desc(ClassLoader classLoader,  
                            MessageDriven md, 
                            AssemblyDescriptor asd, 
                            JonasMessageDriven jMd, 
                            JLinkedList jMDRList, 
                            String fileName)
            throws DeploymentDescException {
        super(classLoader, md,  asd, jMd, jMDRList, fileName);
        // message selector
        selector = (md.getMessageSelector()!=null)?md.getMessageSelector():null;

        // acknowledge mode
        if (md.getAcknowledgeMode()==null) {
            acknowledgeMode = AUTO_ACKNOWLEDGE;
        } else {
            if (md.getAcknowledgeMode().equals("Auto-acknowledge")) {
                acknowledgeMode = AUTO_ACKNOWLEDGE;
            } else  if (md.getAcknowledgeMode().equals("Dups-ok-acknowledge")) {
                acknowledgeMode = DUPS_OK_ACKNOWLEDGE;
            } else {
                throw new DeploymentDescException("Invalid acknowledge-mode content for ejb-name "+ejbName);
            }
        }


        if (md.getMessageDrivenDestination() == null) {
            throw new Error("No message-driven-destination specified for message-driven bean "+ejbName);
        }
        MessageDrivenDestination d = md.getMessageDrivenDestination();
        if (d.getDestinationType().equals("javax.jms.Queue")) {
            destinationType = javax.jms.Queue.class;
        } else  if (d.getDestinationType().equals("javax.jms.Topic")) {
            destinationType = javax.jms.Topic.class;
            isTopicDestination = true;
        } else {
            throw new DeploymentDescException("Invalid destination-type content for ejb-name "+ejbName);
        }
        if (d.getSubscriptionDurability() != null) {
            if (destinationType.equals(javax.jms.Queue.class)) {
                throw new DeploymentDescException("subscription-durability of message-driven-destination for ejb-name "+ejbName+" defined");
            }
            if (d.getSubscriptionDurability().equals("Durable")) {
                subscriptionDurability = SUBS_DURABLE;
            } else  if (d.getSubscriptionDurability().equals("NonDurable")) {
                subscriptionDurability = SUBS_NON_DURABLE;
            } else {
                throw new DeploymentDescException("Invalid subscription-durability content for ejb-name "+ejbName);
            }
        } else {
            // non-durable subscription default value for topic
            if (destinationType.equals(javax.jms.Topic.class)) {
                subscriptionDurability = SUBS_NON_DURABLE;
            }
        }

        // cache onMessage TxAttribute
        for (Iterator i=getMethodDescIterator();i.hasNext();) {
            MethodDesc methd = (MethodDesc)i.next();
            if (methd.getMethod().getName().equals("onMessage")) {
                txAttribute = methd.getTxAttribute();
                break;
            }
        }
    }

}

