/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): Lutris Tecnologies Inc.
 *
 * --------------------------------------------------------------------------
 * $Id: MessageDrivenJMS2_1Desc.java,v 1.5 2004/05/24 04:40:40 ehardesty Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.deployment.api;

import java.util.ListIterator;

import org.objectweb.jonas_ejb.deployment.xml.ActivationConfig;
import org.objectweb.jonas_ejb.deployment.xml.ActivationConfigProperty;
import org.objectweb.jonas_ejb.deployment.xml.AssemblyDescriptor;
import org.objectweb.jonas_ejb.deployment.xml.JonasMessageDriven;
import org.objectweb.jonas_ejb.deployment.xml.MessageDriven;
import org.objectweb.jonas_ejb.deployment.xml.MessageDrivenDestination;
import org.objectweb.jonas_lib.deployment.api.DeploymentDescException;
import org.objectweb.jonas_lib.deployment.xml.JLinkedList;
/**
 * Class to hold meta-information related to a message driven bean
 * @author Philippe Coq
 */
public class MessageDrivenJMS2_1Desc extends MessageDrivenDesc {



    protected MessageDriven messageDriven = null;

    protected JonasMessageDriven jonasMessageDriven = null;

    /**
     * package protected constructor used by API
     */
    MessageDrivenJMS2_1Desc(ClassLoader classLoader,  
                            MessageDriven md, 
                            AssemblyDescriptor asd, 
                            JonasMessageDriven jmd, 
                            JLinkedList jMDRList, 
                            String fileName)
            throws DeploymentDescException {
        super(classLoader, md,  asd, jmd, jMDRList, fileName);
        messageDriven = md;
        jonasMessageDriven = jmd;
        
        // Set values can be from old 2.0 way, ActivationConfig in std xml, 
        //  or ActivationConfig in jonas xml
        
        // message selector
        selector = md.getMessageSelector();
        // acknowledge mode
        if (md.getAcknowledgeMode() == null) {
            acknowledgeMode = AUTO_ACKNOWLEDGE;
        } else {
            if (md.getAcknowledgeMode().equals("Auto-acknowledge")) {
                acknowledgeMode = AUTO_ACKNOWLEDGE;
            } else  if (md.getAcknowledgeMode().equals("Dups-ok-acknowledge")) {
                acknowledgeMode = DUPS_OK_ACKNOWLEDGE;
            } else {
                throw new DeploymentDescException("Invalid acknowledge-mode content for ejb-name "+ejbName);
            }
        }
        MessageDrivenDestination d = md.getMessageDrivenDestination();
        if (d != null && d.getDestinationType() != null) {
            if (d.getDestinationType().equals("javax.jms.Queue")) {
                destinationType = javax.jms.Queue.class;
            } else  if (d.getDestinationType().equals("javax.jms.Topic")) {
                destinationType = javax.jms.Topic.class;
                isTopicDestination = true;
            } else {
                throw new DeploymentDescException("Invalid destination-type content for ejb-name "+ejbName);
            }
            if (d.getSubscriptionDurability() != null) {
                if (destinationType.equals(javax.jms.Queue.class)) {
                    throw new DeploymentDescException("subscription-durability of message-driven-destination for ejb-name "+ejbName+" defined");
                }
                if (d.getSubscriptionDurability().equals("Durable")) {
                    subscriptionDurability = SUBS_DURABLE;
                } else  if (d.getSubscriptionDurability().equals("NonDurable")) {
                    subscriptionDurability = SUBS_NON_DURABLE;
                } else {
                    throw new DeploymentDescException("Invalid subscription-durability content for ejb-name "+ejbName);
                }
            } else {
                // non-durable subscription default value for topic
                if (destinationType.equals(javax.jms.Topic.class)) {
                    subscriptionDurability = SUBS_NON_DURABLE;
                }
            }

        }

        destinationLink = md.getMessageDestinationLink();

        ActivationConfig ac = md.getActivationConfig();
        if( ac != null) {
            configureAC(ac);
        } 
        
        ac = jmd.getActivationConfig();
        if( ac != null) {
            configureAC(ac);
        } 
        
        if (destinationJndiName == null) {
            throw new Error("No destination specified for message-driven bean "+ejbName);
        }

    }

    /**
     * @return the MessageDriven object
     */
    public MessageDriven getMessageDriven() {
        return messageDriven;
    }

    /**
     * @return the MessageDriven object
     */
    public JonasMessageDriven getJonasMessageDriven() {
        return jonasMessageDriven;
    }
    
    private void configureAC(ActivationConfig ac) throws DeploymentDescException{
        try {
            JLinkedList acpl = ac.getActivationConfigPropertyList();
            for (ListIterator lit = acpl.listIterator();lit.hasNext();) {
                ActivationConfigProperty el = (ActivationConfigProperty)lit.next();
                if (el.getActivationConfigPropertyName().equals("destinationType")) {
                    if (el.getActivationConfigPropertyValue().equals("javax.jms.Queue")) {
                        destinationType = javax.jms.Queue.class;
                    } else if (el.getActivationConfigPropertyValue().equals("javax.jms.Topic")) {
                        destinationType = javax.jms.Topic.class;
                        isTopicDestination = true;
                    } else {
                        throw new DeploymentDescException("destinationType value for ejb-name "+ejbName);
                    } 
                } else if (el.getActivationConfigPropertyName().equals("messageSelector")) {
                    selector = el.getActivationConfigPropertyValue();
                }
                else if (el.getActivationConfigPropertyName().equals("acknowledgeMode")) {
                    if (el.getActivationConfigPropertyValue().equals("Auto-acknowledge")) {
                    acknowledgeMode = AUTO_ACKNOWLEDGE;
                    } else  if (el.getActivationConfigPropertyValue().equals("Dups-ok-acknowledge")) {
                        acknowledgeMode = DUPS_OK_ACKNOWLEDGE;
                }
                } else if (el.getActivationConfigPropertyName().equals("subscriptionDurability")) {
                    if (el.getActivationConfigPropertyValue().equals("Durable")) {
                        subscriptionDurability = SUBS_DURABLE;
                    } else {
                        subscriptionDurability = SUBS_NON_DURABLE;
                    }
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

}

