/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A. 
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 *
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer: Florent BENOIT
 * --------------------------------------------------------------------------
 * $Id: PoolParams.java,v 1.2 2003/11/11 20:03:28 ehardesty Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_rar.deployment.xml;

import org.objectweb.jonas_lib.deployment.xml.AbsElement;

/** 
 * This class defines the implementation of the element pool-params
 * 
 * @author Florent Benoit
 */

public class PoolParams extends AbsElement  {

    /**
     * pool-init
     */ 
    private String poolInit = null;

    /**
     * pool-min
     */ 
    private String poolMin = null;

    /**
     * pool-max
     */ 
    private String poolMax = null;

    /**
     * pool-max-age
     */ 
    private String poolMaxAge = null;

    /**
     * pstmt-max
     */ 
    private String pstmtMax = null;


    /**
     * Constructor
     */
    public PoolParams() {
        super();
    }

    /** 
     * Gets the pool-init
     * @return the pool-init
     */
    public String getPoolInit() {
        return poolInit;
    }

    /** 
     * Set the pool-init
     * @param poolInit poolInit
     */
    public void setPoolInit(String poolInit) {
        this.poolInit = poolInit;
    }

    /** 
     * Gets the pool-min
     * @return the pool-min
     */
    public String getPoolMin() {
        return poolMin;
    }

    /** 
     * Set the pool-min
     * @param poolMin poolMin
     */
    public void setPoolMin(String poolMin) {
        this.poolMin = poolMin;
    }

    /** 
     * Gets the pool-max
     * @return the pool-max
     */
    public String getPoolMax() {
        return poolMax;
    }

    /** 
     * Set the pool-max
     * @param poolMax poolMax
     */
    public void setPoolMax(String poolMax) {
        this.poolMax = poolMax;
    }

    /** 
     * Gets the pool-max-age
     * @return the pool-max-age
     */
    public String getPoolMaxAge() {
        return poolMaxAge;
    }

    /** 
     * Set the pool-max-age
     * @param poolMaxAge poolMaxAge
     */
    public void setPoolMaxAge(String poolMaxAge) {
        this.poolMaxAge = poolMaxAge;
    }

    /** 
     * Gets the pstmt-max
     * @return the pstmt-max
     */
    public String getPstmtMax() {
        return pstmtMax;
    }

    /** 
     * Set the pstmt-max
     * @param pstmtMax pstmtMax
     */
    public void setPstmtMax(String pstmtMax) {
        this.pstmtMax = pstmtMax;
    }

    /**
     * Represents this element by it's XML description.
     * @param indent use this indent for prefixing XML representation.
     * @return the XML description of this object.
     */
    public String toXML(int indent) {
        StringBuffer sb = new StringBuffer();
        sb.append(indent(indent));
        sb.append("<pool-params>\n");

        indent += 2;

        // pool-init
        sb.append(xmlElement(poolInit, "pool-init", indent));
        // pool-min
        sb.append(xmlElement(poolMin, "pool-min", indent));
        // pool-max
        sb.append(xmlElement(poolMax, "pool-max", indent));
        // pool-max-age
        sb.append(xmlElement(poolMaxAge, "pool-max-age", indent));
        // pstmt-max
        sb.append(xmlElement(pstmtMax, "pstmt-max", indent));
        indent -= 2;
        sb.append(indent(indent));
        sb.append("</pool-params>\n");

        return sb.toString();
    }
}
