/**
 * JOnAS : Java(TM) OpenSource Application Server
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial Developer : Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: DummyApplication.java,v 1.3 2004/04/19 13:49:53 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.archive;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Vector;
import java.util.jar.Attributes;
import java.util.jar.Manifest;

import org.objectweb.jonas_ws.wsgen.WsGenException;
import org.objectweb.jonas_ws.wsgen.utils.FileUtils;
import org.objectweb.jonas_ws.wsgen.utils.TempRepository;
import org.objectweb.jonas_ws.wsgen.utils.XMLUtils;

/**
 * DummyApplication is a wrapper for auto generated application archive.
 *
 * @author Guillaume Sauthier
 */
public class DummyApplication extends Application {

    /** application filename */
    private String name;

    /**
     * Creates a new Application archive.
     *
     * @param name the file containing the application archive.
     *
     * @throws WsGenException When Init fails
     */
    public DummyApplication(String name) throws WsGenException {
        super(createFileArchive());
        this.name = name;
        init();
    }

    /**
     * Create a new FileArchive
     *
     * @return a new Archive
     *
     * @throws WsGenException When Archive creation fails.
     */
    private static Archive createFileArchive() throws WsGenException {
        TempRepository tr = TempRepository.getInstance();

        try {
            File tmp = tr.createDir();
            File meta = new File(tmp, "META-INF");
            meta.mkdirs();

            File manifest = new File(meta, "MANIFEST.MF");

            Manifest mf = new Manifest();
            mf.getMainAttributes().putValue(Attributes.Name.MANIFEST_VERSION.toString(), "1.0");

            OutputStream os = new FileOutputStream(manifest);
            mf.write(os);
            os.close();

            String jonasRoot = System.getProperty("install.root");
            File appTemplate = new File(new File(jonasRoot), "templates" + File.separator + "wsgen" + File.separator
                    + "application.xml");
            FileUtils.copy(appTemplate, meta);

            return new FileArchive(tmp);
        } catch (IOException ioe) {
            throw new WsGenException(ioe);
        }
    }

    /**
     * Initialize the DummyApplication. Creates modules lists. Overriddes
     * Application.init() behavior
     *
     * @throws WsGenException When Descriptor cannot be parsed.
     */
    protected void init() throws WsGenException {
        // remove dummy modules
        setEjbjars(new Vector());
        setWebapps(new Vector());
        setClients(new Vector());

        loadDescriptors();

        XMLUtils.cleanDummyApplication(getApp());

    }

    /**
     * @return Returns the DummyApplication filename.
     */
    public String getName() {
        return name;
    }

}