/**
 * JOnAS : Java(TM) OpenSource Application Server
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial Developer : Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: WsEndpointGenerator.java,v 1.3 2004/04/19 13:49:54 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.generator;

import org.objectweb.jonas_ws.deployment.api.ServiceDesc;
import org.objectweb.jonas_ws.wsgen.WsGenException;
import org.objectweb.jonas_ws.wsgen.ddmodifier.WebServicesDDModifier;
import org.objectweb.jonas_ws.wsgen.ddmodifier.WsEndpointDDModifier;

/**
 * Generate sources and/or config files for WebServices Endpoint.
 *
 * @author Guillaume Sauthir
 */
public abstract class WsEndpointGenerator extends Generator {

    /** WebService Endpoint description */
    private ServiceDesc service;

    /** Web DD Modifier */
    private WsEndpointDDModifier modifier;

    /** Webservices DD Modifier */
    private WebServicesDDModifier wsModifier;

    /**
     * Creates a new WsEndpointGenerator
     *
     * @param config Generator Configuration
     * @param serviceDesc WebService Endpoint description
     * @param ddm Web DD Modifier
     * @param wsddm webservices.xml DD modifier
     *
     * @throws WsGenException When instanciation fails
     */
    public WsEndpointGenerator(Config config, ServiceDesc serviceDesc, WsEndpointDDModifier ddm,
            WebServicesDDModifier wsddm) throws WsGenException {
        super(config);
        service = serviceDesc;
        modifier = ddm;
        wsModifier = wsddm;
    }
    /**
     * @return Returns the service.
     */
    public ServiceDesc getService() {
        return service;
    }
    /**
     * @return Returns the modifier.
     */
    public WsEndpointDDModifier getModifier() {
        return modifier;
    }
    /**
     * @return Returns the wsModifier.
     */
    public WebServicesDDModifier getWsModifier() {
        return wsModifier;
    }
}