/**
 * JOnAS : Java(TM) OpenSource Application Server
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial Developer : Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: ApplicationModifier.java,v 1.4 2004/04/19 13:49:54 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.modifier;

import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.objectweb.jonas_ws.wsgen.WsGenException;
import org.objectweb.jonas_ws.wsgen.archive.Application;
import org.objectweb.jonas_ws.wsgen.archive.Archive;
import org.objectweb.jonas_ws.wsgen.archive.Client;
import org.objectweb.jonas_ws.wsgen.archive.EjbJar;
import org.objectweb.jonas_ws.wsgen.archive.WebApp;
import org.objectweb.jonas_ws.wsgen.generator.GeneratorFactory;

import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Modify a given Application.
 *
 * @author Guillaume Sauthier
 */
public class ApplicationModifier extends ArchiveModifier {

    /** ejbjar modifiers */
    private List ejbModifiers;

    /** web modifiers */
    private List webModifiers;

    /** client modifiers */
    private List cltModifiers;

    /** Wrapped application */
    private Application app;

    /**
     * Creates a new ApplicationModifier.
     *
     * @param archive the Application J2EE archive
     */
    public ApplicationModifier(Application archive) {
        super(archive);
        app = archive;
        init();
    }

    /**
     * initialize modifier
     */
    private void init() {
        // create lists
        ejbModifiers = new Vector();
        webModifiers = new Vector();
        cltModifiers = new Vector();

        // fill ejbjar list
        for (Iterator i = app.getEjbJars(); i.hasNext();) {
            EjbJar ejbjar = (EjbJar) i.next();
            ejbModifiers.add(new EjbJarModifier(ejbjar));
        }

        // fill webapp list
        for (Iterator i = app.getWebApps(); i.hasNext();) {
            WebApp webapp = (WebApp) i.next();
            webModifiers.add(new WebAppModifier(webapp));
        }

        // fill client list
        for (Iterator i = app.getClients(); i.hasNext();) {
            Client client = (Client) i.next();
            cltModifiers.add(new ClientModifier(client));
        }
    }

    /**
     * Modify the current archive and return a modified archive.
     *
     * @return a modified archive.
     *
     * @throws WsGenException When modifications fails
     */
    public Archive modify() throws WsGenException {

        getLogger().log(BasicLevel.INFO, "Processing Application " + app.getName());

        /**
         * Modify inner modules
         */
        for (Iterator i = webModifiers.iterator(); i.hasNext();) {
            WebAppModifier wm = (WebAppModifier) i.next();
            Archive a = wm.modify();
            getLogger().log(BasicLevel.DEBUG, "Modifying WebApp '" + a.getName() + "'");
            app.addFile(a.getRootFile());
        }

        for (Iterator i = cltModifiers.iterator(); i.hasNext();) {
            ClientModifier cm = (ClientModifier) i.next();
            Archive a = cm.modify();
            getLogger().log(BasicLevel.DEBUG, "Modifying Client '" + a.getName() + "'");
            app.addFile(a.getRootFile());
        }

        for (Iterator i = ejbModifiers.iterator(); i.hasNext();) {
            EjbJarModifier em = (EjbJarModifier) i.next();
            Archive a = em.modify();
            getLogger().log(BasicLevel.DEBUG, "Modifying EjbJar '" + a.getName() + "'");
            app.addFile(a.getRootFile());
        }

        return save(GeneratorFactory.getInstance().getConfiguration(), "apps" + File.separator + app.getName());

    }
}