/**
 * JOnAS : Java(TM) OpenSource Application Server
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial Developer : Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: ArchiveModifier.java,v 1.5 2004/04/19 13:49:54 sauthieg Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen.modifier;

import java.io.File;

import org.objectweb.jonas_ws.wsgen.WsGenException;
import org.objectweb.jonas_ws.wsgen.archive.Archive;
import org.objectweb.jonas_ws.wsgen.archive.FileArchive;
import org.objectweb.jonas_ws.wsgen.archive.J2EEArchive;
import org.objectweb.jonas_ws.wsgen.archive.JarArchive;
import org.objectweb.jonas_ws.wsgen.generator.Config;
import org.objectweb.jonas_ws.wsgen.utils.ArchiveStorer;
import org.objectweb.jonas_ws.wsgen.utils.DirStorer;
import org.objectweb.jonas_ws.wsgen.utils.JarStorer;

import org.objectweb.jonas.common.Log;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Modify a J2EEArchive
 *
 * @author Guillaume Sauthier
 */
public abstract class ArchiveModifier {

    /**
     * J2EEArchive to be modified
     */
    private J2EEArchive archive;

    /** logger */
    private static Logger logger = Log.getLogger(Log.JONAS_WSGEN_PREFIX);

    /**
     * Creates a new ArchiveModifier object.
     *
     * @param archive rchive to be modified
     */
    public ArchiveModifier(J2EEArchive archive) {
        this.archive = archive;
    }

    /**
     * Modify the current archive and return a modified archive.
     *
     * @return a modified archive.
     *
     * @throws WsGenException When Archive modification fails
     */
    public abstract Archive modify() throws WsGenException;

    /**
     * Save the curernt archive usin given configuration and given out filename.
     *
     * @param config configuration to use
     * @param outname filename
     * @param archive archive to be saved
     *
     * @return the save Archive
     *
     * @throws WsGenException is save fails.
     */
    protected static Archive save(Config config, String outname, J2EEArchive archive) throws WsGenException {
        ArchiveStorer storer = null;
        File endfile = new File(config.getOut(), outname);
        Archive out = null;

        if (config.getSaveMode() == Config.PACKED) {
            logger.log(BasicLevel.DEBUG, "Saving '" + endfile + "' in packed form...");
            storer = new JarStorer(archive, endfile);
            storer.store();
            out = new JarArchive(endfile);
        } else {
            logger.log(BasicLevel.DEBUG, "Saving '" + endfile + "' in unpacked form...");
            storer = new DirStorer(archive, endfile);
            storer.store();
            out = new FileArchive(endfile);
        }

        return out;

    }

    /**
     * Save the curernt archive usin given configuration and given out filename.
     *
     * @param config configuration to use
     * @param outname filename
     *
     * @return the save Archive
     *
     * @throws WsGenException is save fails.
     */
    protected Archive save(Config config, String outname) throws WsGenException {
        return save(config, outname, archive);
    }
    /**
     * @return Returns the logger.
     */
    public static Logger getLogger() {
        return logger;
    }
}